@@ -22,9 +22,14 @@
 #include "mainwindow.h"
 #include "networkAccess.h"
 #include "utility.h"
+#include "themes.h"
+#include "mainwindow.h"
+#include "versionInfo.h"
 
 #include <QFileDialog>
 #include <QFile>
+#include <QDesktopServices>
+#include <QClipboard>
 
 configure::configure( const Context& ctx ) :
 	m_ctx( ctx ),
@@ -35,20 +40,22 @@ configure::configure( const Context& ctx ) :
 	m_engines( m_ctx.Engines() ),
 	m_tablePresetOptions( *m_ui.tableWidgetConfigurePresetOptions,m_ctx.mainWidget().font() ),
 	m_tableUrlToDefaultEngine( *m_ui.tableWidgetConfigureUrl,m_ctx.mainWidget().font() ),
+	m_tableDefaultDownloadOptions( *m_ui.tableWidgetEnginesDefaultOptions,m_ctx.mainWidget().font() ),
 	m_presetOptions( m_ctx,m_settings ),
-	m_downloadDefaultOptions( m_ctx )
+	m_downloadDefaultOptions( m_ctx,"downloadDefaultOptions.json" ),
+	m_downloadEngineDefaultOptions( m_ctx,"downloadEngineDefaultOptions.json" )
 {
 	m_ui.tableWidgetConfigureUrl->setColumnWidth( 0,180 ) ;
 
-	m_ui.widgetConfigureManageUrl->setVisible( false ) ;
-
 	m_ui.lineEditConfigureScaleFactor->setEnabled( m_settings.enabledHighDpiScaling() ) ;
 
 	m_ui.tabWidgetConfigure->setCurrentIndex( 0 ) ;
 
-	settings::darkModes modes ;
+	auto themesFolderPath = m_engines.engineDirPaths().themePath() ;
 
-	modes.setComboBox( *m_ui.comboBoxConfigureDarkTheme,m_settings.darkMode() ) ;
+	themes ths( themesFolderPath ) ;
+
+	ths.setComboBox( *m_ui.comboBoxConfigureDarkTheme,m_settings.themeName() ) ;
 
 	auto cc = static_cast< void ( QComboBox::* )( int ) >( &QComboBox::currentIndexChanged ) ;
 
@@ -62,11 +69,32 @@ configure::configure( const Context& ctx ) :
 		m_tablePresetOptions.selectRow( c,p,0 ) ;
 	} ) ;
 
-	connect( m_ui.comboBoxConfigureDarkTheme,cc,[ this,modes = std::move( modes ) ]( int index ){
+	m_tableDefaultDownloadOptions.connect( &QTableWidget::currentItemChanged,[ this ]( QTableWidgetItem * c,QTableWidgetItem * p ){
+
+		m_tablePresetOptions.selectRow( c,p,0 ) ;
+	} ) ;
+
+	m_ui.pbOpenThemeFolder->setIcon( QIcon( ":/json" ) ) ;
+
+	connect( m_ui.pbOpenThemeFolder,&QPushButton::clicked,[ themesFolderPath ](){
+
+		QDesktopServices::openUrl( QUrl( "file:///" + themesFolderPath,QUrl::TolerantMode ) ) ;
+	} ) ;
+
+	m_ui.pbOpenBinFolder->setIcon( QIcon( ":/executable" ) ) ;
+
+	connect( m_ui.pbOpenBinFolder,&QPushButton::clicked,[ this,themesFolderPath ](){
+
+		const auto& m = m_engines.engineDirPaths().binPath() ;
+
+		QDesktopServices::openUrl( QUrl( "file:///" + m,QUrl::TolerantMode ) ) ;
+	} ) ;
+
+	connect( m_ui.comboBoxConfigureDarkTheme,cc,[ this,ths = std::move( ths ) ]( int index ){
 
 		if( index != -1 ){
 
-			m_settings.setDarkMode( modes.unTranslatedAt( index ) ) ;
+			m_settings.setThemeName( ths.unTranslatedAt( index ) ) ;
 		}
 	} ) ;
 
@@ -90,12 +118,7 @@ configure::configure( const Context& ctx ) :
 
 		QMenu m ;
 
-		connect( m.addAction( tr( "Add" ) ),&QAction::triggered,[ this ](){
-
-			m_ui.widgetConfigureManageUrl->setVisible( true ) ;
-		} ) ;
-
-		connect( m.addAction( tr( "Delete" ) ),&QAction::triggered,[ this ](){
+		connect( m.addAction( tr( "Remove" ) ),&QAction::triggered,[ this ](){
 
 			auto m = m_tableUrlToDefaultEngine.currentRow() ;
 
@@ -110,16 +133,97 @@ configure::configure( const Context& ctx ) :
 		m.exec( QCursor::pos() ) ;
 	} ) ;
 
-	connect( m_ui.pbConfigureManageUrlCancel,&QPushButton::clicked,[ this ](){
+	m_tableDefaultDownloadOptions.connect( &QTableWidget::customContextMenuRequested,[ this ]( QPoint ){
 
-		m_ui.widgetConfigureManageUrl->setVisible( false ) ;
+		QMenu m ;
+
+		connect( m.addAction( tr( "Set As Default" ) ),&QAction::triggered,[ this ](){
+
+			auto m = m_tableDefaultDownloadOptions.currentRow() ;
+
+			if( m != -1 ){
+
+				auto mm = m_ui.cbConfigureEngines->currentText() ;
+
+				const auto& s = m_ctx.Engines().getEngineByName( mm ) ;
+
+				if( s ){
+					auto obj = m_tableDefaultDownloadOptions.stuffAt( m ) ;
+
+					m_downloadEngineDefaultOptions.setAsDefault( obj ) ;
+
+					this->populateOptionsTable( s ) ;
+				}
+			}
+		} ) ;
+
+		connect( m.addAction( tr( "Remove" ) ),&QAction::triggered,[ this ](){
+
+			auto m = m_tableDefaultDownloadOptions.currentRow() ;
+
+			if( m != -1 ){
+
+				auto obj = m_tableDefaultDownloadOptions.stuffAt( m ) ;
+				m_downloadEngineDefaultOptions.remove( obj ) ;
+
+				auto mm = m_ui.cbConfigureEngines->currentText() ;
+
+				const auto& s = m_ctx.Engines().getEngineByName( mm ) ;
+
+				this->populateOptionsTable( s ) ;
+			}
+		} ) ;
+
+		connect( m.addAction( tr( "Copy" ) ),&QAction::triggered,[ this ](){
+
+			auto s = m_tableDefaultDownloadOptions.currentRow() ;
+
+			auto m = QApplication::clipboard() ;
+
+			if( s != -1 && m ){
+
+				auto obj = m_tableDefaultDownloadOptions.stuffAt( s ) ;
+
+				auto str = obj.value( "options" ).toString() ;
+
+				if( !str.isEmpty() ){
+
+					m->setText( str ) ;
+				}
+			}
+		} ) ;
+
+		m.exec( QCursor::pos() ) ;
+	} ) ;
+
+	connect( m_ui.pbAddDefaultDownloadOption,&QPushButton::clicked,[ this ](){
+
+		auto m = m_ui.lineEditAddDefaultDownloadOption->text() ;
+
+		if( !m.isEmpty() ){
+
+			auto mm = m_ui.cbConfigureEngines->currentText() ;
+
+			const auto& s = m_ctx.Engines().getEngineByName( mm ) ;
+
+			if( s ){
+
+				auto obj = m_downloadEngineDefaultOptions.addOpt( "no",s->name(),m ) ;
+
+				m_tableDefaultDownloadOptions.add( { "no",m },std::move( obj ) ) ;
+
+				m_tableDefaultDownloadOptions.selectLast() ;
+
+				m_ui.lineEditAddDefaultDownloadOption->clear() ;
+			}
+		}
 	} ) ;
 
 	connect( m_ui.pbConfigureManageUrl,&QPushButton::clicked,[ this ](){
 
 		auto a = m_ui.lineEditConfigureManageUrl->text() ;
 		auto b = m_ui.lineEditConfigureManageOptions->text() ;
-		auto c = m_ui.cbConfigureEngines->currentText() ;
+		auto c = m_ui.cbConfigureEnginesUrlManager->currentText() ;
 
 		if( !a.isEmpty() ){
 
@@ -136,8 +240,6 @@ configure::configure( const Context& ctx ) :
 			m_tableUrlToDefaultEngine.add( { a,b },std::move( obj ) ) ;
 
 			m_tableUrlToDefaultEngine.selectLast() ;
-
-			m_ui.widgetConfigureManageUrl->setVisible( false ) ;
 		}
 	} ) ;
 
@@ -155,16 +257,43 @@ configure::configure( const Context& ctx ) :
 			}
 		} ) ;
 
+		connect( m.addAction( tr( "Copy" ) ),&QAction::triggered,[ this ](){
+
+			auto row = m_tablePresetOptions.currentRow() ;
+
+			if( row != -1 ){
+
+				auto mm = QApplication::clipboard() ;
+
+				if( mm ){
+
+					mm->setText( m_tablePresetOptions.item( row,1 ).text() ) ;
+				}
+			}
+		} ) ;
+
 		m.exec( QCursor::pos() ) ;
 	} ) ;
 
-	connect( m_ui.cbConfigureEngines,cc,[ this,modes = std::move( modes ) ]( int index ){
+	connect( m_ui.cbConfigureEngines,cc,[ this ]( int index ){
 
 		if( index != -1 ){
 
 			auto m = m_ui.cbConfigureEngines->itemText( index ) ;
 
-			this->setEngineOptions( m ) ;
+			this->populateOptionsTable( m_engines.getEngineByName( m ) ) ;
+
+			this->setEngineOptions( m,engineOptions::options ) ;
+		}
+	} ) ;
+
+	connect( m_ui.cbConfigureEnginesUrlManager,cc,[ this ]( int index ){
+
+		if( index != -1 ){
+
+			auto m = m_ui.cbConfigureEngines->itemText( index ) ;
+
+			this->setEngineOptions( m,engineOptions::url ) ;
 		}
 	} ) ;
 
@@ -181,7 +310,7 @@ configure::configure( const Context& ctx ) :
 	connect( m_ui.pbConfigureQuit,&QPushButton::clicked,[ this ](){
 
 		this->saveOptions() ;
-		m_tabManager.basicDownloader().appQuit() ;
+		m_ctx.mainWindow().quitApp() ;
 	} ) ;
 
 	connect( m_ui.pbConfigureSave,&QPushButton::clicked,[ this ](){
@@ -189,35 +318,76 @@ configure::configure( const Context& ctx ) :
 		this->saveOptions() ;
 	} ) ;
 
-	connect( m_ui.cbConfigureShowThumbnails,&QCheckBox::stateChanged,[]( int ){
+	connect( m_ui.cbShowTrayIcon,&QCheckBox::stateChanged,[ this ]( int s ){
 
-	} ) ;
+		auto checked = s == Qt::CheckState::Checked ;
 
-	connect( m_ui.cbUseSystemVersionIfAvailable,&QCheckBox::stateChanged,[]( int ){
+		m_settings.setshowTrayIcon( checked ) ;
 
+		m_ctx.mainWindow().showTrayIcon( checked ) ;
 	} ) ;
 
 	connect( m_ui.pbConfigureAddAPlugin,&QPushButton::clicked,[ this ](){
 
 		auto m = QFileDialog::getOpenFileName( &m_ctx.mainWidget(),tr( "Select An Engine File" ),utility::homePath() ) ;
 
-		if( !m.isEmpty() ){
+		if( m.isEmpty() ){
 
-			auto d = engines::file( m,m_ctx.logger() ).readAll() ;
+			return ;
+		}
 
-			if( !d.isEmpty() ){
+		auto id = utility::sequentialID() ;
 
-				auto name = m_ctx.Engines().addEngine( d,util::split( m,'/',true ).last() ) ;
+		auto d = engines::file( m,m_ctx.logger() ).readAll() ;
 
-				if( !name.isEmpty() ){
+		if( d.isEmpty() ){
 
-					auto& t = m_ctx.TabManager() ;
+			return ;
+		}
 
-					t.basicDownloader().setAsActive() ;
-					t.setDefaultEngines() ;
-					m_ctx.versionInfo().check( m_ctx.Engines().defaultEngine( name ) ) ;
+		auto name = m_ctx.Engines().addEngine( d,util::split( m,'/',true ).last(),id ) ;
+
+		if( name.isEmpty() ){
+
+			return ;
+		}
+
+		auto& t = m_ctx.TabManager() ;
+
+		t.basicDownloader().setAsActive() ;
+
+		const auto& engine = m_ctx.Engines().getEngineByName( name ) ;
+
+		if( engine ){
+
+			class woof : public versionInfo::idone
+			{
+			public:
+				woof( const Context& ctx,QString de ) :
+					m_ctx( ctx ),m_defaultEngine( std::move( de ) )
+				{
 				}
-			}
+				void operator()() override
+				{
+					if( m_success ){
+
+						utility::setDefaultEngine( m_ctx,m_defaultEngine ) ;
+					}
+				}
+				void failed() override
+				{
+					m_success = false ;
+				}
+			private:
+				const Context& m_ctx ;
+				QString m_defaultEngine ;
+				bool m_success = true ;
+			} ;
+
+			auto& m = m_ctx.getVersionInfo() ;
+
+			m.check( { engine.value(),id },
+				 { util::types::type_identity< woof >(),m_ctx,std::move( name ) } ) ;
 		}
 	} ) ;
 
@@ -237,7 +407,9 @@ configure::configure( const Context& ctx ) :
 
 		connect( &m,&QMenu::triggered,[ & ]( QAction * ac ){
 
-			m_ctx.Engines().removeEngine( ac->objectName() ) ;
+			auto id = utility::sequentialID() ;
+
+			m_ctx.Engines().removeEngine( ac->objectName(),id ) ;
 
 			auto& t = m_ctx.TabManager() ;
 
@@ -259,7 +431,30 @@ configure::configure( const Context& ctx ) :
 
 		if( !m.isEmpty() ){
 
-			this->downloadFromGitHub( m_ctx.Engines().defaultEngine( m ) ) ;
+			auto id = utility::sequentialID() ;
+
+			if( m == m_ctx.appName() ){
+
+				class meaw : public networkAccess::status
+				{
+				public:
+					meaw( const Context& ctx ) : m_ctx( ctx )
+					{
+					}
+					void done()
+					{
+						m_ctx.TabManager().enableAll() ;
+					}
+				private:
+					const Context& m_ctx ;
+				} ;
+
+				networkAccess::Status s{ util::types::type_identity< meaw >(),m_ctx } ;
+
+				m_ctx.network().updateMediaDownloader( id,std::move( s ) ) ;
+			}else{
+				this->downloadFromGitHub( { m_ctx.Engines().defaultEngine( m,id ),id } ) ;
+			}
 		}
 	} ) ;
 
@@ -282,6 +477,8 @@ configure::configure( const Context& ctx ) :
 			m_ctx.mainWindow().retranslateUi() ;
 
 			m_tabManager.reTranslateUi() ;
+
+			m_ctx.logger().reTranslateLogWindow() ;
 		}
 	} ) ;
 
@@ -293,10 +490,7 @@ configure::configure( const Context& ctx ) :
 		this->showOptions() ;
 	} ) ;
 
-	m_ui.pbConfigureDownloadPath->setIcon( [](){
-
-		return QIcon( ":folder" ) ;
-	}() ) ;
+	m_ui.pbConfigureDownloadPath->setIcon( QIcon( ":/folder" ) ) ;
 
 	connect( m_ui.pbConfigureDownloadPath,&QPushButton::clicked,[ this ](){
 
@@ -319,11 +513,9 @@ configure::configure( const Context& ctx ) :
 
 		if( s ){
 
-			auto m = s->defaultDownLoadCmdOptions().join( " " ) ;
+			m_downloadEngineDefaultOptions.removeAll( s->name() ) ;
 
-			m_ui.lineEditConfigureDownloadOptions->setText( m ) ;
-
-			m_settings.setEngineDefaultDownloadOptions( s->name(),m ) ;
+			this->populateOptionsTable( s ) ;
 		}
 	} ) ;
 
@@ -337,7 +529,30 @@ configure::configure( const Context& ctx ) :
 
 	m_ui.cbUseSystemVersionIfAvailable->setChecked( m_settings.useSystemProvidedVersionIfAvailable() ) ;
 
-	m_ui.cbUseSystemVersionIfAvailable->setEnabled( utility::platformIsLinux() ) ;
+	m_ui.cbAutoSaveNotDownloadedMedia->setChecked( m_settings.autoSavePlaylistOnExit() ) ;
+
+	m_ui.lineEditConfigureTextEncoding->setText( m_settings.textEncoding() ) ;
+
+	m_ui.cbCheckForUpdates->setChecked( m_settings.checkForUpdates() ) ;
+
+	m_ui.cbShowTrayIcon->setChecked( m_settings.showTrayIcon() ) ;
+
+	connect( m_ui.cbCheckForUpdates,&QCheckBox::stateChanged,[ this ]( int e ){
+
+		auto s = static_cast< Qt::CheckState >( e ) ;
+
+		if( s == Qt::Checked ){
+
+			m_ctx.getVersionInfo().checkForUpdates() ;
+		}
+	} ) ;
+
+	if( utility::platformIsLikeWindows() ){
+
+		m_ui.cbUseSystemVersionIfAvailable->setEnabled( !m_settings.portableVersion() ) ;
+	}else{
+		m_ui.cbUseSystemVersionIfAvailable->setEnabled( utility::platformIsLinux() ) ;
+	}
 
 	m_ui.lineEditConfigureMaximuConcurrentDownloads->setText( QString::number( m_settings.maxConcurrentDownloads() ) ) ;
 
@@ -353,21 +568,29 @@ void configure::retranslateUi()
 {
 	this->resetMenu() ;
 
-	settings::darkModes().setComboBox( *m_ui.comboBoxConfigureDarkTheme,m_settings.darkMode() ) ;
+	themes().setComboBox( *m_ui.comboBoxConfigureDarkTheme,m_settings.themeName() ) ;
 }
 
 void configure::downloadFromGitHub( const engines::Iterator& iter )
 {
-	m_ctx.versionInfo().network().download( iter ) ;
+	m_ctx.network().download( iter,{ true,false } ) ;
 }
 
 void configure::tabEntered()
 {
 	m_menu.clear() ;
 
+	auto ac = m_menu.addAction( m_ctx.appName() ) ;
+
+	ac->setObjectName( m_ctx.appName() ) ;
+
+	ac->setEnabled( utility::platformIsWindows() ) ;
+
+	m_menu.addSeparator() ;
+
 	for( const auto& it : m_ctx.Engines().getEngines() ){
 
-		if( !it.downloadUrl().isEmpty() ){
+		if( it.validDownloadUrl() ){
 
 			auto ac = m_menu.addAction( it.name() ) ;
 
@@ -382,6 +605,44 @@ void configure::tabEntered()
 	m_menu.addAction( tr( "Cancel" ) ) ;
 
 	m_ui.pbConfigureDownload->setMenu( &m_menu ) ;
+
+	auto mm = m_ui.cbConfigureEngines->currentText() ;
+
+	this->populateOptionsTable( m_engines.getEngineByName( mm ) ) ;
+}
+
+void configure::populateOptionsTable( const util::result_ref< const engines::engine& >& s )
+{
+	m_tableDefaultDownloadOptions.clear() ;
+
+	if( s ){
+
+		m_downloadEngineDefaultOptions.forEach( [ s,this ]( const configure::downloadDefaultOptions::optsEngines& opts,QJsonObject obj ){
+
+			if( s->name() == opts.engine ){
+
+				m_tableDefaultDownloadOptions.add( { opts.inuse,opts.options },std::move( obj ) ) ;
+			}
+
+			return false ;
+		} ) ;
+
+		if( m_tableDefaultDownloadOptions.rowCount() == 0 ){
+
+			const auto& e = s->defaultDownLoadCmdOptions() ;
+
+			if( !e.isEmpty() ){
+
+				auto b = e.join( " " ) ;
+
+				auto obj = m_downloadEngineDefaultOptions.addOpt( "yes",s->name(),b ) ;
+
+				m_tableDefaultDownloadOptions.add( { "yes",b },std::move( obj ) ) ;
+			}
+		}
+
+		m_tableDefaultDownloadOptions.selectLast() ;
+	}
 }
 
 void configure::tabExited()
@@ -392,18 +653,44 @@ void configure::tabExited()
 void configure::updateEnginesList( const QStringList& e )
 {
 	auto& cb = *m_ui.cbConfigureEngines ;
+	auto& xb = *m_ui.cbConfigureEnginesUrlManager ;
 
 	cb.clear() ;
+	xb.clear() ;
 
 	for( const auto& it : e ){
 
 		cb.addItem( it ) ;
+		xb.addItem( it ) ;
 
-		this->setEngineOptions( it ) ;
+		this->setEngineOptions( it,engineOptions::both ) ;
 	}
 
 	cb.setCurrentIndex( 0 ) ;
-	this->setEngineOptions( cb.currentText() ) ;
+	xb.setCurrentIndex( 0 ) ;
+
+	this->setEngineOptions( cb.currentText(),engineOptions::both ) ;
+}
+
+QString configure::engineDefaultDownloadOptions( const QString& engineName )
+{
+	QString options ;
+
+	using mm = configure::downloadDefaultOptions::optsEngines ;
+
+	m_downloadEngineDefaultOptions.forEach( [ & ]( const mm& opts,const QJsonObject& ){
+
+		if( opts.engine == engineName && opts.inuse == "yes" ){
+
+			options = opts.options ;
+
+			return true ;
+		}else{
+			return false ;
+		}
+	} ) ;
+
+	return options ;
 }
 
 void configure::setDownloadOptions( int row,tableWidget& table )
@@ -414,6 +701,7 @@ void configure::setDownloadOptions( int row,tableWidget& table )
 void configure::saveOptions()
 {
 	m_downloadDefaultOptions.save() ;
+	m_downloadEngineDefaultOptions.save() ;
 
 	auto m = m_ui.cbConfigureShowThumbnails->isChecked() ;
 
@@ -424,6 +712,9 @@ void configure::saveOptions()
 	m_settings.setDownloadFolder( m_ui.lineEditConfigureDownloadPath->text() ) ;
 	m_settings.setShowVersionInfoWhenStarting( m_ui.cbConfigureShowVersionInfo->isChecked() ) ;
 	m_settings.setUseSystemProvidedVersionIfAvailable( m_ui.cbUseSystemVersionIfAvailable->isChecked() ) ;
+	m_settings.setAutoSavePlaylistOnExit( m_ui.cbAutoSaveNotDownloadedMedia->isChecked() ) ;
+	m_settings.setCheckForUpdates( m_ui.cbCheckForUpdates->isChecked() ) ;
+	m_settings.setTextEncoding( m_ui.lineEditConfigureTextEncoding->text() ) ;
 
 	auto s = m_ui.lineEditConfigureMaximuConcurrentDownloads->text() ;
 
@@ -452,10 +743,6 @@ void configure::saveOptions()
 
 	if( ss ){
 
-		auto m = m_ui.lineEditConfigureDownloadOptions->text() ;
-
-		m_settings.setEngineDefaultDownloadOptions( ss->name(),m ) ;
-
 		if( !ss->cookieArgument().isEmpty() ){
 
 			m_settings.setCookieFilePath( ss->name(),m_ui.lineEditConfigureCookiePath->text() ) ;
@@ -466,45 +753,52 @@ void configure::saveOptions()
 	m_ctx.TabManager().resetMenu() ;
 }
 
-void configure::setEngineOptions( const QString& e )
+void configure::setEngineOptions( const QString& e,engineOptions tab )
 {
 	const auto& s = m_engines.getEngineByName( e ) ;
 
 	if( s ){
 
-		const auto& engineName = s->name() ;
+		auto _setUpUrl = [ & ](){
 
-		m_tableUrlToDefaultEngine.clear() ;
+			const auto& engineName = s->name() ;
 
-		m_downloadDefaultOptions.forEach( [ this,&engineName ]( const downloadDefaultOptions::opts& e,QJsonObject obj ){
+			m_tableUrlToDefaultEngine.clear() ;
 
-			if( engineName.isEmpty() || engineName == e.engine ){
+			m_downloadDefaultOptions.forEach( [ this,&engineName ]( const downloadDefaultOptions::opts& e,QJsonObject obj ){
 
-				m_tableUrlToDefaultEngine.add( { e.url,e.downloadOptions },std::move( obj ) ) ;
-			}
+				if( engineName.isEmpty() || engineName == e.engine ){
 
-			return false ;
-		} ) ;
+					m_tableUrlToDefaultEngine.add( { e.url,e.downloadOptions },std::move( obj ) ) ;
+				}
 
-		m_tableUrlToDefaultEngine.selectLast() ;
+				return false ;
+			} ) ;
 
-		auto m = m_settings.engineDefaultDownloadOptions( s->name() ) ;
+			m_tableUrlToDefaultEngine.selectLast() ;
+		} ;
 
-		if( m.isEmpty() ){
+		auto _setUpDownloadOptions = [ & ](){
 
-			const auto& e = s->defaultDownLoadCmdOptions() ;
-			m_settings.setEngineDefaultDownloadOptions( s->name(),e.join( " " ) ) ;
-			m_ui.lineEditConfigureDownloadOptions->setText( e.join( " " ) ) ;
+			auto enable = !s->cookieArgument().isEmpty() ;
+
+			m_ui.lineEditConfigureCookiePath->setText( m_settings.cookieFilePath( s->name() ) ) ;
+			m_ui.lineEditConfigureCookiePath->setEnabled( enable ) ;
+			m_ui.pbConfigureCookiePath->setEnabled( enable ) ;
+			m_ui.labelPathToCookieFile->setEnabled( enable ) ;
+		} ;
+
+		if( tab == engineOptions::url ){
+
+			_setUpUrl() ;
+
+		}else if( tab == engineOptions::options ){
+
+			_setUpDownloadOptions() ;
 		}else{
-			m_ui.lineEditConfigureDownloadOptions->setText( m ) ;
+			_setUpUrl() ;
+			_setUpDownloadOptions() ;
 		}
-
-		auto enable = !s->cookieArgument().isEmpty() ;
-
-		m_ui.lineEditConfigureCookiePath->setText( m_settings.cookieFilePath( s->name() ) ) ;
-		m_ui.lineEditConfigureCookiePath->setEnabled( enable ) ;
-		m_ui.pbConfigureCookiePath->setEnabled( enable ) ;
-		m_ui.labelPathToCookieFile->setEnabled( enable ) ;
 	}
 }
 
@@ -521,7 +815,14 @@ void configure::savePresetOptions()
 		auto uiName = table.item( i,0 )->text() ;
 		auto options = table.item( i,1 )->text() ;
 
-		m_presetOptions.add( uiName,options ) ;
+		const auto& e = m_tablePresetOptions.stuffAt( i ) ;
+
+		if( e.isEmpty() ){
+
+			m_presetOptions.add( uiName,options ) ;
+		}else{
+			m_presetOptions.add( e,options ) ;
+		}
 	}
 }
 
@@ -529,7 +830,20 @@ void configure::showOptions()
 {
 	this->presetOptionsForEach( [ this ]( const QString& uiName,const QString& options ){
 
-		m_tablePresetOptions.add( { uiName,options } ) ;
+		if( uiName == "Default" ){
+
+			m_tablePresetOptions.add( { tr( "Default" ),options },"Default" ) ;
+
+		}else if( uiName == "Best-audio" ){
+
+			m_tablePresetOptions.add( { tr( "Best-audio" ),options },"Best-audio" ) ;
+
+		}else if( uiName == "Best-audiovideo" ){
+
+			m_tablePresetOptions.add( { tr( "Best-audiovideo" ),options },"Best-audiovideo" ) ;
+		}else{
+			m_tablePresetOptions.add( { uiName,options } ) ;
+		}
 	} )  ;
 }
 
@@ -560,6 +874,10 @@ void configure::resetMenu()
 	m_ui.cbConfigureLanguage->setCurrentIndex( index ) ;
 }
 
+void configure::exiting()
+{
+}
+
 void configure::enableAll()
 {
 	const auto& s = m_engines.getEngineByName( m_ui.cbConfigureEngines->currentText() ) ;
@@ -573,6 +891,24 @@ void configure::enableAll()
 		m_ui.labelPathToCookieFile->setEnabled( enable ) ;
 	}
 
+	m_ui.label_6->setEnabled( true ) ;
+	m_ui.label_3->setEnabled( true ) ;
+	m_ui.label_4->setEnabled( true ) ;
+	m_ui.label_5->setEnabled( true ) ;
+	m_ui.lineEditConfigureTextEncoding->setEnabled( true ) ;
+	m_ui.labelConfigureTextEncoding->setEnabled( true ) ;
+	m_ui.cbCheckForUpdates->setEnabled( true ) ;
+	m_ui.pbOpenThemeFolder->setEnabled( true ) ;
+	m_ui.pbOpenBinFolder->setEnabled( true ) ;
+	m_ui.cbConfigureEnginesUrlManager->setEnabled( true ) ;
+	m_ui.tableWidgetConfigureUrl->setEnabled( true ) ;
+	m_ui.labelConfigureEngines_2->setEnabled( true ) ;
+	m_ui.pbConfigureManageUrl->setEnabled( true ) ;
+	m_ui.lineEditConfigureManageUrl->setEnabled( true ) ;
+	m_ui.lineEditConfigureManageOptions->setEnabled( true ) ;
+	m_ui.lineEditAddDefaultDownloadOption->setEnabled( true ) ;
+	m_ui.tableWidgetEnginesDefaultOptions->setEnabled( true ) ;
+	m_ui.pbAddDefaultDownloadOption->setEnabled( true ) ;
 	m_ui.tableWidgetConfigurePresetOptions->setEnabled( true ) ;
 	m_ui.lineEditConfigurePresetOptions->setEnabled( true ) ;
 	m_ui.lineEditConfigureUiName->setEnabled( true ) ;
@@ -580,8 +916,6 @@ void configure::enableAll()
 	m_ui.labelConfigureOptionsPresetOptiions->setEnabled( true ) ;
 	m_ui.pbConfigureAddToPresetList->setEnabled( true ) ;
 	m_ui.pbConfigureEngineDefaultOptions->setEnabled( true ) ;
-	m_ui.lineEditConfigureDownloadOptions->setEnabled( true ) ;
-	m_ui.labelConfigureOptions->setEnabled( true ) ;
 	m_ui.cbConfigureEngines->setEnabled( true ) ;
 	m_ui.labelConfigureEngines->setEnabled( true ) ;
 	m_ui.pbConfigureSave->setEnabled( true ) ;
@@ -590,6 +924,7 @@ void configure::enableAll()
 	m_ui.pbConfigureDownload->setEnabled( true ) ;
 	m_ui.labelConfigureTheme->setEnabled( true ) ;
 	m_ui.cbConfigureShowVersionInfo->setEnabled( true ) ;
+	m_ui.cbAutoSaveNotDownloadedMedia->setEnabled( true ) ;
 	m_ui.cbConfigureLanguage->setEnabled( true ) ;
 	m_ui.labelConfigureLanguage->setEnabled( true ) ;
 	m_ui.lineEditConfigureDownloadPath->setEnabled( true ) ;
@@ -603,17 +938,42 @@ void configure::enableAll()
 	m_ui.pbConfigureRemoveAPlugin->setEnabled( true ) ;
 	m_ui.cbConfigureShowThumbnails->setEnabled( true ) ;
 	m_ui.labelMaximumConcurrentDownloads->setEnabled( true ) ;
+	m_ui.cbShowTrayIcon->setEnabled( true ) ;
 
 	if( m_settings.enabledHighDpiScaling() ){
 
 		m_ui.lineEditConfigureScaleFactor->setEnabled( true ) ;
 	}
 
-	m_ui.cbUseSystemVersionIfAvailable->setEnabled( utility::platformIsLinux() ) ;
+	if( utility::platformIsLikeWindows() ){
+
+		m_ui.cbUseSystemVersionIfAvailable->setEnabled( !m_settings.portableVersion() ) ;
+	}else{
+		m_ui.cbUseSystemVersionIfAvailable->setEnabled( utility::platformIsLinux() ) ;
+	}
 }
 
 void configure::disableAll()
 {
+	m_ui.label_3->setEnabled( false ) ;
+	m_ui.label_4->setEnabled( false ) ;
+	m_ui.label_5->setEnabled( false ) ;
+	m_ui.label_6->setEnabled( false ) ;
+	m_ui.lineEditConfigureTextEncoding->setEnabled( false ) ;
+	m_ui.labelConfigureTextEncoding->setEnabled( false ) ;
+	m_ui.cbCheckForUpdates->setEnabled( false ) ;
+	m_ui.pbOpenThemeFolder->setEnabled( false ) ;
+	m_ui.labelConfigureEngines_2->setEnabled( false ) ;
+	m_ui.cbConfigureEnginesUrlManager->setEnabled( false ) ;
+	m_ui.lineEditConfigureManageUrl->setEnabled( false ) ;
+	m_ui.tableWidgetConfigureUrl->setEnabled( false ) ;
+	m_ui.pbConfigureManageUrl->setEnabled( false ) ;
+	m_ui.pbOpenBinFolder->setEnabled( false ) ;
+	m_ui.lineEditAddDefaultDownloadOption->setEnabled( false ) ;
+	m_ui.tableWidgetEnginesDefaultOptions->setEnabled( false ) ;
+	m_ui.cbShowTrayIcon->setEnabled( false ) ;
+	m_ui.pbAddDefaultDownloadOption->setEnabled( false ) ;
+	m_ui.lineEditConfigureManageOptions->setEnabled( false ) ;
 	m_ui.tableWidgetConfigurePresetOptions->setEnabled( false ) ;
 	m_ui.lineEditConfigurePresetOptions->setEnabled( false ) ;
 	m_ui.lineEditConfigureUiName->setEnabled( false ) ;
@@ -625,8 +985,7 @@ void configure::disableAll()
 	m_ui.lineEditConfigureCookiePath->setEnabled( false ) ;
 	m_ui.pbConfigureCookiePath->setEnabled( false ) ;
 	m_ui.pbConfigureEngineDefaultOptions->setEnabled( false ) ;
-	m_ui.lineEditConfigureDownloadOptions->setEnabled( false ) ;
-	m_ui.labelConfigureOptions->setEnabled( false ) ;
+	m_ui.cbAutoSaveNotDownloadedMedia->setEnabled( false ) ;
 	m_ui.cbConfigureEngines->setEnabled( false ) ;
 	m_ui.labelConfigureEngines->setEnabled( false ) ;
 	m_ui.pbConfigureSave->setEnabled( false ) ;
@@ -780,8 +1139,8 @@ QByteArray configure::presetOptions::defaultData()
 ])R";
 }
 
-configure::downloadDefaultOptions::downloadDefaultOptions( const Context& ctx ) :
-	m_path( ctx.Engines().engineDirPaths().dataPath( "downloadDefaultOptions.json" ) )
+configure::downloadDefaultOptions::downloadDefaultOptions( const Context& ctx,const QString& name ) :
+	m_path( ctx.Engines().engineDirPaths().dataPath( name ) )
 {
 	QJsonParseError err ;
 
@@ -807,22 +1166,56 @@ void configure::downloadDefaultOptions::save()
 	f.write( QJsonDocument( m_array ).toJson( QJsonDocument::Indented ) ) ;
 }
 
-QJsonObject configure::downloadDefaultOptions::add( const QString& url,
-						    const QString& opts,
-						    const QString& engineName )
+bool configure::downloadDefaultOptions::isEmpty( const QString& m )
 {
+	for( const auto& it : util::asConst( m_array ) ){
+
+		auto s = it.toObject().value( "engineName" ).toString() ;
+
+		if( s == m ){
+
+			return false ;
+		}
+	}
+
+	return true ;
+}
+
+QJsonObject configure::downloadDefaultOptions::addOpt( const QString& inUse,
+						       const QString& engineName,
+						       const QString& options )
+{
+	for( const auto& it : util::asConst( m_array ) ){
+
+		auto obj = it.toObject() ;
+
+		auto a = obj.value( "options" ).toString() ;
+		auto b = obj.value( "engineName" ).toString() ;
+
+		if( a == options && b == engineName ){
+
+			return obj ;
+		}
+	}
+
 	QJsonObject obj ;
 
-	obj.insert( "comparator","contains" ) ;
-	obj.insert( "downloadOption",opts ) ;
-	obj.insert( "engine",engineName ) ;
-	obj.insert( "url",url ) ;
+	obj.insert( "default",inUse ) ;
+	obj.insert( "options",options ) ;
+	obj.insert( "engineName",engineName ) ;
 
 	m_array.append( obj ) ;
 
 	return obj ;
 }
 
+QJsonObject configure::downloadDefaultOptions::add( const QString& url,
+						    const QString& opts,
+						    const QString& engineName )
+{
+	return this->add( { "contains",opts,engineName,url } ) ;
+}
+
 QJsonObject configure::downloadDefaultOptions::add( const configure::downloadDefaultOptions::opts& e )
 {
 	QJsonObject obj ;
@@ -847,4 +1240,89 @@ void configure::downloadDefaultOptions::remove( const QJsonObject& e )
 			break ;
 		}
 	}
+
+	auto engineName = e.value( "engineName" ) ;
+
+	for( int i = 0 ; i < m_array.size() ; i++ ){
+
+		auto obj = m_array[ i ].toObject() ;
+
+		auto name = obj.value( "engineName" ) ;
+
+		if( name == engineName ){
+
+			auto s = obj.value( "default" ) ;
+
+			if( s == "yes" ){
+
+				return ;
+			}
+		}
+	}
+
+	for( int i = 0 ; i < m_array.size() ; i++ ){
+
+		auto obj = m_array[ i ].toObject() ;
+
+		auto name = obj.value( "engineName" ) ;
+
+		if( name == engineName ){
+
+			obj.insert( "default","yes" ) ;
+
+			m_array[ i ] = obj ;
+
+			break ;
+		}
+	}
+}
+
+void configure::downloadDefaultOptions::removeAll( const QString& e )
+{
+	while( [ this,&e ](){
+
+		for( int i = 0 ; i < m_array.size() ; i++ ){
+
+			auto s = m_array[ i ].toObject().value( "engineName" ) ;
+
+			if( s == e ){
+
+				m_array.removeAt( i ) ;
+				return true ;
+			}
+		}
+
+		return false ;
+	}() ){} ;
+}
+
+QJsonObject configure::downloadDefaultOptions::setAsDefault( const QJsonObject& e )
+{
+	auto engineName = e.value( "engineName" ).toString() ;
+	auto options = e.value( "options" ).toString() ;
+
+	QJsonObject xbj ;
+
+	for( int i = 0 ; i < m_array.size() ; i++ ){
+
+		auto obj = m_array[ i ].toObject() ;
+		auto engine = obj.value( "engineName" ).toString() ;
+		auto opts = obj.value( "options" ).toString() ;
+
+		if( engine == engineName ){
+
+			if( options == opts ){
+
+				xbj = obj ;
+
+				obj.insert( "default","yes" ) ;
+			}else{
+				obj.insert( "default","no" ) ;
+			}
+
+			m_array[ i ] = obj ;
+		}
+	}
+
+	return xbj ;
 }
