@@ -24,6 +24,7 @@
 #include <QStandardPaths>
 #include <QPlainTextEdit>
 #include <QJsonObject>
+#include <QJsonArray>
 #include <QJsonDocument>
 #include <QDir>
 #include <QProcess>
@@ -35,12 +36,20 @@
 
 #include "logger.h"
 #include "util.hpp"
+#include "utils/threads.hpp"
 
 class tableWidget ;
 class settings ;
+class Context ;
+
+namespace utility {
+	class uiIndex ;
+}
 
 class engines{
 public:
+	static bool filePathIsValid( const QFileInfo& ) ;
+
 	class file
 	{
 	public:
@@ -55,7 +64,40 @@ public:
 			    QJsonDocument::JsonFormat = QJsonDocument::Indented ) ;
 		QByteArray readAll() ;
 		QStringList readAllAsLines() ;
+		template< typename Function >
+		static void readAll( const QString& filePath,Logger& logger,Function function )
+		{
+			struct result
+			{
+				bool success ;
+				QByteArray data ;
+			};
+
+			utils::qthread::run( [ filePath ]()->result{
+
+				QFile f( filePath ) ;
+
+				if( f.open( QIODevice::ReadOnly ) ){
+
+					return { true,f.readAll() } ;
+				}else{
+					return { false,{} } ;
+				}
+
+			},[ &logger,filePath,function = std::move( function ) ]( result r ){
+
+				if( r.success ){
+
+					function( true,r.data ) ;
+				}else{
+					engines::file( filePath,logger ).failToOpenForReading() ;
+					function( false,r.data ) ;
+				}
+			} ) ;
+		}
 	private:
+		void failToOpenForReading() ;
+		void failToOpenForWriting() ;
 		QString m_filePath ;
 		QFile m_file ;
 		Logger& m_logger ;
@@ -82,14 +124,42 @@ public:
 		{
 			return m_dataPath ;
 		}
+		const QString& updatePath() const
+		{
+			return m_updatePath ;
+		}
+		const QString& tmp() const
+		{
+			return m_tmp ;
+		}
+		const QString updateNewPath() const
+		{
+			return m_updateNewPath ;
+		}
+		QString tmp( const QString& e ) const
+		{
+			return m_tmp + "/" +  e ;
+		}
+		QString updatePath( const QString& e ) const
+		{
+			return m_updatePath + "/" +  e ;
+		}
 		QString dataPath( const QString& e ) const
 		{
 			return m_dataPath + "/" + e ;
 		}
+		QString subscriptionsArchiveFilePath() const
+		{
+			return m_dataPath + "/subscriptions_archive_file.txt" ;
+		}
 		QString binPath( const QString& e ) const
 		{
 			return m_binPath + "/" + e ;
 		}
+		QString themePath() const
+		{
+			return m_basePath + "/themes" ;
+		}
 		QString enginePath( const QString& e ) const
 		{
 			return m_enginePath + "/" + e ;
@@ -100,6 +170,9 @@ public:
 		QString m_enginePath ;
 		QString m_basePath ;
 		QString m_dataPath ;
+		QString m_updatePath ;
+		QString m_updateNewPath ;
+		QString m_tmp ;
 	};
 
 	class engine
@@ -129,7 +202,9 @@ public:
 				}
 				bool valid()
 				{
-					return QFile::exists( m_exe ) ;
+					QFileInfo info( m_exe ) ;
+
+					return engines::filePathIsValid( info ) ;
 				}
 			private:
 				QStringList m_args ;
@@ -165,6 +240,16 @@ public:
 			{
 				return m_realExe ;
 			}
+			void updateRealExe( const QString& e )
+			{
+				if( m_exe.at( 0 ) == m_realExe ){
+
+					m_exe[ 0 ] = e ;
+					m_realExe  = e ;
+				}else{
+					m_realExe = e ;
+				}
+			}
 		private:
 			QStringList m_exe ;
 			QString m_realExe ;
@@ -181,7 +266,9 @@ public:
 				finishedState( const Args& a ) :
 				        m_success( a.success() ),
 				        m_cancelled( a.cancelled() ),
-				        m_duration( a.duration() )
+					m_duration( a.duration() ),
+					m_errorCode( a.exitCode() ),
+					m_exitStatus( a.exitStatus() )
 				{
 				}
 				bool success() const
@@ -192,16 +279,30 @@ public:
 				{
 				        return m_cancelled ;
 				}
-				int duration() const
+				qint64 duration() const
 				{
 				        return m_duration ;
 				}
+				int errorCode() const
+				{
+					return m_errorCode ;
+				}
+				QProcess::ExitStatus exitStatus() const
+				{
+					return m_exitStatus ;
+				}
 			private:
 				bool m_success ;
 				bool m_cancelled ;
-				int m_duration ;
+				qint64 m_duration ;
+				int m_errorCode ;
+				QProcess::ExitStatus m_exitStatus ;
 			};
 
+			enum class errors{ unknownUrl,noNetwork,unknownFormat } ;
+			static QString errorString( const engine::engine::functions::finishedState&,
+						    engines::engine::functions::errors,
+						    const QString& ) ;
 			static QString processCompleteStateText( const engine::engine::functions::finishedState& ) ;
 
 			class timer
@@ -210,23 +311,29 @@ public:
 				static bool timerText( const QString& e ) ;
 				static QString timerText() ;
 				static QString startTimerText() ;
-				static QString stringElapsedTime( int ) ;
-				static QString duration( int ) ;
+				static QString stringElapsedTime( qint64 ) ;
+				static QString duration( qint64 ) ;
 				static int toSeconds( const QString& ) ;
-				int elapsedTime() ;
+				static qint64 currentTime() ;
+				qint64 elapsedTime() ;
 				QString stringElapsedTime() ;
+				void reset() ;
 			private:
-				qint64 m_startTime = QDateTime().currentMSecsSinceEpoch() ;
+				qint64 m_startTime = engines::engine::functions::timer::currentTime() ;
 			};
 
 			class preProcessing
 			{
 			public:
 				preProcessing() ;
-				preProcessing( const QString& ) ;
+				preProcessing( const QByteArray&,int = 16 ) ;
+
 				static QByteArray processingText() ;
+				void reset() ;
 				const QByteArray& text() ;
+				const QByteArray& text( const QByteArray& ) ;
 			private:
+				int m_maxCounter = 16 ;
 				int m_counter = 0 ;
 				QByteArray m_counterDots ;
 				QByteArray m_txt ;
@@ -238,9 +345,9 @@ public:
 			public:
 				static QByteArray processingText() ;
 				postProcessing() ;
-				postProcessing( const QString& ) ;
+				postProcessing( const QByteArray& ) ;
 
-				const QByteArray& text( const QString& ) ;
+				const QByteArray& text( const QByteArray& ) ;
 			private:
 				int m_counter = 0 ;
 				QByteArray m_counterDots ;
@@ -250,7 +357,7 @@ public:
 
 			class filter{
 			public:
-				filter( const QString& quality,const engines::engine& engine ) ;
+				filter( const QString& quality,const engines::engine& engine,int ) ;
 				virtual const QByteArray& operator()( const Logger::Data& e ) ;
 				virtual ~filter() ;
 				const engines::engine& engine() const ;
@@ -260,6 +367,8 @@ public:
 				engines::engine::functions::preProcessing m_processing ;
 				QString m_quality ;
 				const engines::engine& m_engine ;
+				QByteArray m_tmp ;
+				int m_processId ;
 			} ;
 
 			class DataFilter{
@@ -279,17 +388,93 @@ public:
 
 			virtual ~functions() ;
 
-			virtual std::vector< QStringList > mediaProperties( const QByteArray& ) ;
+			virtual const QProcessEnvironment& processEnvironment() const ;
+
+			class mediaInfo
+			{
+			public:
+				mediaInfo( const QStringList& u,const QString& i,const QString& e,const QString& r,const QString& n ) :
+					m_url( u ),m_id( i ),m_extension( e ),m_resolution( r ),m_info( n )
+				{
+				}
+				mediaInfo( const QString& i,const QString& e,const QString& r,const QString& n ) :
+					m_id( i ),m_extension( e ),m_resolution( r ),m_info( n )
+				{
+				}
+				QStringList toStringList() const
+				{
+					return { m_id,m_extension,m_resolution,m_info } ;
+				}
+				QJsonObject toqJsonObject() const
+				{
+					QJsonObject obj ;
+
+					QJsonArray arr ;
+
+					for( const auto& it : m_url ){
+
+						arr.append( it ) ;
+					}
+
+					obj.insert( "urls",arr ) ;
+					obj.insert( "id",m_id ) ;
+					obj.insert( "extension",m_extension ) ;
+					obj.insert( "resolution",m_resolution ) ;
+					obj.insert( "info",m_info ) ;
+
+					return obj ;
+				}
+			private:
+				QStringList m_url ;
+				QString m_id ;
+				QString m_extension ;
+				QString m_resolution ;
+				QString m_info ;
+			} ;
+
+			virtual std::vector< engines::engine::functions::mediaInfo > mediaProperties( const QByteArray& ) ;
+
+			virtual std::vector< engines::engine::functions::mediaInfo > mediaProperties( const QJsonArray& ) ;
+
+			virtual void updateOutPutChannel( QProcess::ProcessChannel& ) const ;
 
 			virtual bool breakShowListIfContains( const QStringList& ) ;
 
-			virtual engines::engine::functions::DataFilter Filter( const QString& ) ;
+			virtual bool supportsShowingComments() ;
+
+			virtual bool updateVersionInfo() ;
+
+			virtual void setTextEncondig( const QString&,QStringList& opts ) ;
+
+			virtual QString updateCmdPath( const QString& ) ;
+
+			virtual engines::engine::functions::DataFilter Filter( int,const QString& ) ;
 
 			virtual void runCommandOnDownloadedFile( const QString&,const QString& ) ;
 
 			virtual QString commandString( const engines::engine::exeArgs::cmd& ) ;
 
-			virtual QStringList dumpJsonArguments() ;
+			virtual QStringList horizontalHeaderLabels() const ;
+
+			virtual void updateEnginePaths( const Context&,QString& filePath,QString& exeBinPath,QString& exeFolderPath ) ;
+
+			virtual bool parseOutput( Logger::Data&,const QByteArray&,int,bool ) ;
+
+			virtual bool likeYtdlp() ;
+
+			virtual void updateLocalOptions( QStringList& ) ;
+
+			struct onlineVersion
+			{
+				QString stringVersion ;
+				util::version version ;
+			};
+
+			virtual engines::engine::functions::onlineVersion versionInfoFromGithub( const QByteArray& ) ;
+
+			virtual bool foundNetworkUrl( const QString& ) ;
+
+			virtual void renameArchiveFolder( const QString& ) ;
 
 			QString updateTextOnCompleteDownlod( const QString& uiText,
 							     const QString& downloadingOptions,
@@ -310,19 +495,31 @@ public:
 			{
 				const QString& quality ;
 				const QStringList& userOptions ;
-				const QString& indexAsString ;
+				const utility::uiIndex& uiIndex ;
+				const QString& playlist ;
+				const QString& playlist_count ;
+				const QString& playlist_id ;
+				const QString& playlist_title ;
+				const QString& playlist_uploader ;
+				const QString& playlist_uploader_id ;
+				const QString& n_entries ;
 				QStringList& urls ;
 				QStringList& ourOptions ;
 			};
 
 			virtual void updateDownLoadCmdOptions( const engines::engine::functions::updateOpts& ) ;
 
-			functions( settings&,const engines::engine& ) ;
+			virtual void updateGetPlaylistCmdOptions( QStringList& ) ;
+
+			virtual void updateCmdOptions( QStringList& ) ;
+
+			functions( settings&,const engines::engine&,const QProcessEnvironment& e ) ;
 			settings& Settings() const ;
 			const engines::engine& engine() const ;
 		private:
 			settings& m_settings ;
 			const engines::engine& m_engine ;
+			const QProcessEnvironment& m_processEnvironment ;
 		} ;
 
 		engine( Logger& l ) ;
@@ -332,12 +529,14 @@ public:
 			const QString& name,
 			const QString& versionArgument,
 			int line,
-			int position ) ;
+			int position,
+			int id ) ;
 
 		engine( Logger& logger,
 			const enginePaths& ePaths,
 			const util::Json& json,
-			const engines& engines ) ;
+			const engines& engines,
+			int id ) ;
 
 		static QString mediaAlreadInArchiveText()
 		{
@@ -349,6 +548,48 @@ public:
 			return m_name ;
 		}
 
+		void updateLocalOptions( QStringList& opts ) const
+		{
+			m_functions->updateLocalOptions( opts ) ;
+		}
+
+		template< typename Context,typename Function >
+		void updateVersionInfo( const Context& ctx,Function function ) const
+		{
+			if( m_functions->updateVersionInfo() ){
+
+				const auto& engine = *this ;
+
+				if( engine.versionInfo().valid() ){
+
+					function() ;
+				}else{
+					ctx.TabManager().disableAll() ;
+
+					const auto& exe = engine.exePath() ;
+					QStringList args{ engine.versionArgument() } ;
+
+					engines::engine::exeArgs::cmd cmd( exe,args ) ;
+
+					this->setPermissions( cmd.exe() ) ;
+
+					utils::qprocess::run( cmd.exe(),cmd.args(),[ &ctx,&engine,function = std::move( function ) ]( const utils::qprocess::outPut& e ){
+
+						if( e.success() ){
+
+							engine.setVersionString( e.stdOut ) ;
+						}
+
+						ctx.TabManager().enableAll() ;
+
+						function() ;
+					} ) ;
+				}
+			}else{
+				function() ;
+			}
+		}
+
 		template< typename backend,typename ... Args >
 		void setBackend( const engines& engines,Args&& ... args )
 		{
@@ -359,6 +600,9 @@ public:
 
 			this->updateOptions() ;
 		}
+
+		QString updateCmdPath( const QString& e ) const ;
+
 		const QString& commandName() const ;
 
 		bool breakShowListIfContains( const QStringList& e ) const ;
@@ -368,6 +612,7 @@ public:
 			return m_versionArgument ;
 		}
 
+		QString setVersionString( const QString& data ) const ;
 		QString versionString( const QString& data ) const ;
 
 		const util::version& versionInfo() const
@@ -382,9 +627,17 @@ public:
 		{
 			return m_downloadUrl ;
 		}
-		QStringList dumpJsonArguments() const
+		bool validDownloadUrl() const
 		{
-			return m_functions->dumpJsonArguments() ;
+			return m_downloadUrl.startsWith( "https://api.github.com" ) ;
+		}		
+		void setTextEncondig( QStringList& opts ) const
+		{
+			m_functions->setTextEncondig( m_encodingArgument,opts ) ;
+		}
+		const QProcessEnvironment& processEnvironment() const
+		{
+			return m_functions->processEnvironment() ;
 		}
 		void processData( Logger::Data& outPut,const QByteArray& data,int id,bool readableJson ) const
 		{
@@ -394,6 +647,10 @@ public:
 		{
 			m_functions->processData( outPut,data,id,readableJson ) ;
 		}
+		void renameArchiveFolder( const QString& e ) const
+		{
+			return m_functions->renameArchiveFolder( e ) ;
+		}
 		QString commandString( const engines::engine::exeArgs::cmd& cmd ) const
 		{
 			return m_functions->commandString( cmd ) ;
@@ -406,9 +663,13 @@ public:
 		{
 			return m_defaultDownLoadCmdOptions ;
 		}
-		engines::engine::functions::DataFilter filter( const QString& quality ) const
+		const QStringList& dumpJsonArguments() const
 		{
-			return m_functions->Filter( quality ) ;
+			return m_dumpJsonArguments ;
+		}
+		engines::engine::functions::DataFilter filter( int processId,const QString& quality ) const
+		{
+			return m_functions->Filter( processId,quality ) ;
 		}
 		QString updateTextOnCompleteDownlod( const QString& uiText,
 						     const QString& bkText,
@@ -425,14 +686,66 @@ public:
 		{
 			m_functions->sendCredentials( credentials,exe ) ;
 		}
-		std::vector< QStringList > mediaProperties( const QByteArray& e ) const
+		std::vector< engines::engine::functions::mediaInfo > mediaProperties( const QByteArray& e ) const
 		{
 			return m_functions->mediaProperties( e ) ;
 		}
+		void updateGetPlaylistCmdOptions( QStringList& e ) const
+		{
+			m_functions->updateGetPlaylistCmdOptions( e ) ;
+		}
+		void updateCmdOptions( QStringList& e ) const
+		{
+			m_functions->updateCmdOptions( e ) ;
+		}
+		bool archiveContainsFolder() const
+		{
+			return m_archiveContainsFolder ;
+		}
+		bool parseOutput( Logger::Data& e,const QByteArray& s,int id,bool m ) const
+		{
+			return m_functions->parseOutput( e,s,id,m ) ;
+		}
+		bool foundNetworkUrl( const QString& s ) const
+		{
+			return m_functions->foundNetworkUrl( s ) ;
+		}
+		engines::engine::functions::onlineVersion versionInfoFromGithub( const QByteArray& e ) const
+		{
+			return m_functions->versionInfoFromGithub( e ) ;
+		}
+		QStringList horizontalHeaderLabels() const
+		{
+			return m_functions->horizontalHeaderLabels() ;
+		}
+		void updateOutPutChannel( QProcess::ProcessChannel& s ) const
+		{
+			m_functions->updateOutPutChannel( s ) ;
+		}
+		std::vector< engines::engine::functions::mediaInfo > mediaProperties( const QJsonArray& e ) const
+		{
+			return m_functions->mediaProperties( e ) ;
+		}
+		void updateEnginePaths( const Context& ctx,QString& filePath,QString& exeBinPath,QString& exeFolderPath ) const
+		{
+			m_functions->updateEnginePaths( ctx,filePath,exeBinPath,exeFolderPath ) ;
+		}
 		const QStringList& defaultListCmdOptions() const
 		{
 			return m_defaultListCmdOptions ;
 		}
+		const QStringList& defaultCommentsCmdOptions() const
+		{
+			return m_defaultCommentsCmdOptions ;
+		}
+		const QStringList& defaultSubstitlesCmdOptions() const
+		{
+			return m_defaultSubstitlesCmdOptions ;
+		}
+		const QStringList& defaultSubtitleDownloadOptions() const
+		{
+			return m_defaultSubtitleDownloadOptions ;
+		}
 		const QStringList& skiptLineWithText() const
 		{
 			return m_skiptLineWithText ;
@@ -469,10 +782,6 @@ public:
 		{
 			return m_playListUrlPrefix ;
 		}
-		const QStringList& playListIdArguments() const
-		{
-			return m_playListIdArguments ;
-		}
 		const QString& playlistItemsArgument() const
 		{
 			return m_playlistItemsArgument ;
@@ -485,6 +794,10 @@ public:
 		{
 			return m_controlStructure ;
 		}
+		bool canDownloadMediaPart() const
+		{
+			return this->name().contains( "yt-dlp" ) ;
+		}
 		bool usingPrivateBackend() const
 		{
 			return m_usingPrivateBackend ;
@@ -493,6 +806,10 @@ public:
 		{
 			return m_valid ;
 		}
+		bool supportShowingComments() const
+		{
+			return m_functions->supportsShowingComments() ;
+		}
 		bool canDownloadPlaylist() const
 		{
 			return m_canDownloadPlaylist ;
@@ -503,7 +820,8 @@ public:
 		}
 		bool backendExists() const
 		{
-			return QFile::exists( m_exePath.realExe() ) ;
+			QFileInfo m( m_exePath.realExe() ) ;
+			return m.exists() && m.isFile() ;
 		}
 		bool mainEngine() const
 		{
@@ -522,23 +840,35 @@ public:
 			return m_broken ;
 		}
 	private:
+		void setPermissions( const QString& ) const ;
+
 		void updateOptions() ;
+
+		void parseMultipleCmdArgs( Logger& logger,const engines& engines,int ) ;
+
+		void parseMultipleCmdArgs( QStringList&,
+					   const QString&,
+					   Logger& logger,
+					   const enginePaths& ePaths,
+					   const engines& engines,
+					   int ) ;
+
 		mutable util::version m_version ;
 		QJsonObject m_jsonObject ;
 		std::unique_ptr< engines::engine::functions > m_functions ;
 		int m_line ;
 		int m_position ;
 		bool m_valid ;
-		bool m_usingPrivateBackend ;
+		bool m_usingPrivateBackend = false ;
 		bool m_canDownloadPlaylist ;
 		bool m_likeYoutubeDl ;
 		bool m_mainEngine ;
+		bool m_archiveContainsFolder = false ;
 		bool m_replaceOutputWithProgressReport ;
 		mutable bool m_broken = false ;
 		QString m_versionArgument ;
 		QString m_name ;
 		QString m_commandName ;
-		QString m_commandNameWindows ;
 		QString m_userName ;
 		QString m_password ;
 		QString m_exeFolderPath ;
@@ -548,26 +878,31 @@ public:
 		QString m_playlistItemsArgument ;
 		QString m_batchFileArgument ;
 		QString m_cookieArgument ;
-		QStringList m_playListIdArguments ;
+		QString m_encodingArgument ;
+		QStringList m_dumpJsonArguments ;
 		QStringList m_splitLinesBy ;
 		QStringList m_removeText ;
 		QStringList m_skiptLineWithText ;
 		QStringList m_defaultDownLoadCmdOptions ;
 		QStringList m_defaultListCmdOptions ;
+		QStringList m_defaultCommentsCmdOptions ;
+		QStringList m_defaultSubstitlesCmdOptions ;
+		QStringList m_defaultSubtitleDownloadOptions ;
+
 		QJsonObject m_controlStructure ;
 
-		exeArgs m_exePath ;
+		engines::engine::exeArgs m_exePath ;
 	};
 	settings& Settings() const;
 	QString findExecutable( const QString& exeName ) const ;
 	const QProcessEnvironment& processEnvironment() const ;
-	QString addEngine( const QByteArray& data,const QString& path ) ;
-	void removeEngine( const QString& name ) ;
+	QString addEngine( const QByteArray& data,const QString& path,int ) ;
+	void removeEngine( const QString& name,int ) ;
 	QStringList enginesList() const ;
-	const engine& defaultEngine( const QString& ) const ;
+	const engine& defaultEngine( const QString&,int ) const ;
 	util::result_ref< const engines::engine& > getEngineByName( const QString& name ) const ;
 	const enginePaths& engineDirPaths() const ;
-	engines( Logger&,settings& ) ;
+	engines( Logger&,settings&,int ) ;
 	void openUrls( tableWidget&,int row ) const ;
 	void openUrls( tableWidget&,int row,const engines::engine& ) const ;
 	void openUrls( const QString& path ) const ;
@@ -575,14 +910,16 @@ public:
 	class Iterator
 	{
 	public:
-		Iterator( const std::vector< engines::engine >& engines ) :
+		Iterator( const std::vector< engines::engine >& engines,int id ) :
 			m_maxCounter( engines.size() ),
-			m_engines( &engines )
+			m_engines( &engines ),
+			m_id( id )
 		{
 		}
-		Iterator( const engines::engine& engine ) :
+		Iterator( const engines::engine& engine,int id ) :
 			m_maxCounter( 1 ),
-			m_engine( &engine )
+			m_engine( &engine ),
+			m_id( id )
 		{
 		}
 		size_t size() const
@@ -608,17 +945,23 @@ public:
 				return ( *m_engines )[ m_counter ] ;
 			}
 		}
+		int id() const
+		{
+			return m_id ;
+		}
 	private:
 		size_t m_counter = 0 ;
 		size_t m_maxCounter ;
 		const engines::engine * m_engine = nullptr ;
 		const std::vector< engines::engine > * m_engines = nullptr ;
+		int m_id ;
 	} ;
 
 	const std::vector< engine >& getEngines() const ;
 	engines::Iterator getEnginesIterator() const ;
+	void setDefaultEngine( const QString& ) ;
 private:
-	void updateEngines( bool ) ;
+	void updateEngines( bool,int ) ;
 	Logger& m_logger ;
 	settings& m_settings ;
 	std::vector< engine > m_backends ;
