@@ -18,13 +18,14 @@
  */
 
 #include "generic.h"
+#include "../utility.h"
+#include "../context.hpp"
 
 generic::generic( const engines& e,const engines::engine& s,QJsonObject& ) :
-	engines::engine::functions( e.Settings(),s )
+	engines::engine::functions( e.Settings(),s,e.processEnvironment() )
 {
 }
 
 generic::~generic()
 {
 }
-
