@@ -25,14 +25,15 @@
 
 #include <QFileDialog>
 #include <QClipboard>
+#include <QMetaObject>
 
 class playlistdownloader::customOptions
 {
 public:
 	customOptions( QStringList&& opts,
 		       const QString& downloadArchivePath,
-		       settings& settings,
-		       const engines::engine& engine ) :
+		       const engines::engine& engine,
+		       const Context& ctx ) :
 		m_options( std::move( opts ) )
 	{
 		utility::arguments Opts( m_options ) ;
@@ -45,31 +46,29 @@ public:
 
 		if( m_breakOnExisting || m_skipOnExisting ){
 
-			auto s = settings.engineDefaultDownloadOptions( engine.name() ) ;
+			auto s = ctx.TabManager().Configure().engineDefaultDownloadOptions( engine.name() ) ;
+
+			if( ctx.Settings().useInternalArchiveFile() ){
+
+				QFile file( downloadArchivePath ) ;
+
+				if( file.open( QIODevice::ReadOnly ) ){
+
+					m_archiveFileData = file.readAll() ;
+				}
+			}
+
 			auto ss = util::splitPreserveQuotes( s ) ;
 
 			auto mm = utility::arguments( ss ).hasValue( "--download-archive" ) ;
 
-			if( mm.isEmpty() ){
-
-				mm = downloadArchivePath ;
-			}
-
-			if( !mm.isEmpty() ){
-
-				if( utility::isRelativePath( mm ) ){
-
-					mm = settings.downloadFolder() + "/" + mm ;
-				}
-			}
-
 			if( !mm.isEmpty() && QFile::exists( mm ) ){
 
 				QFile file( mm ) ;
 
 				if( file.open( QIODevice::ReadOnly ) ){
 
-					m_archiveFileData = file.readAll() ;
+					m_archiveFileData.append( "\n" + file.readAll() ) ;
 				}
 			}
 		}
@@ -92,7 +91,7 @@ public:
 
 			return false ;
 		}else{
-			return m_archiveFileData.contains( e.toUtf8() + "\n" ) ;
+			return m_archiveFileData.contains( e.toUtf8() ) ;
 		}
 	}
 	bool breakOnExisting() const
@@ -118,12 +117,17 @@ playlistdownloader::playlistdownloader( Context& ctx ) :
 	m_ui( m_ctx.Ui() ),
 	m_mainWindow( m_ctx.mainWidget() ),
 	m_tabManager( m_ctx.TabManager() ),
-	m_table( *m_ui.tableWidgetPl,m_ctx.mainWidget().font(),1 ),
+	m_table( *m_ui.tableWidgetPl,m_ctx.mainWidget().font(),1,m_settings.textAlignment() ),
 	m_subscriptionTable( *m_ui.tableWidgetPlDownloaderSubscription,m_ctx.mainWidget().font() ),
 	m_ccmd( m_ctx,*m_ui.pbPLCancel,m_settings ),
 	m_defaultVideoThumbnailIcon( m_settings.defaultVideoThumbnailIcon( settings::tabName::playlist ) ),
+	m_banner( m_table ),
 	m_subscription( m_ctx,m_subscriptionTable,*m_ui.widgetPlDownloader )
 {
+	qRegisterMetaType< PlNetworkData >() ;
+
+	m_ui.pbPLChangeTableSize->setToolTip( tr( "Hide Controls" ) ) ;
+
 	m_ui.tableWidgetPlDownloaderSubscription->setColumnWidth( 0,180 ) ;
 
 	m_ui.labelPlSubscriptionListOptions->setText( tr( "Get List Options:" ).replace( ":","" ) ) ;
@@ -132,6 +136,21 @@ playlistdownloader::playlistdownloader( Context& ctx ) :
 
 	this->resetMenu() ;
 
+	connect( m_ui.pbClearArchiveFile,&QPushButton::clicked,[ this ](){
+
+		auto m = m_ctx.Engines().engineDirPaths().subscriptionsArchiveFilePath() ;
+		QFile f( m ) ;
+		f.open( QIODevice::WriteOnly | QIODevice::Truncate ) ;
+		f.write( "" ) ;
+	} ) ;
+
+	m_ui.cbUseInternalArchiveFile->setChecked( m_ctx.Settings().useInternalArchiveFile() ) ;
+
+	connect( m_ui.cbUseInternalArchiveFile,&QCheckBox::toggled,[ this ]( bool e ){
+
+		m_ctx.Settings().setUseInternalArchiveFile( e ) ;
+	} ) ;
+
 	m_ui.lineEditPLDownloadRange->setText( m_settings.playlistRangeHistoryLastUsed() ) ;
 
 	connect( m_ui.pbPlSubscriptionDone,&QPushButton::clicked,[ this ](){
@@ -157,6 +176,21 @@ playlistdownloader::playlistdownloader( Context& ctx ) :
 		}
 	} ) ;
 
+	connect( m_ui.pbPLDownloadOptions,&QPushButton::clicked,[ this ](){
+
+		auto& t = m_ctx.TabManager().Configure() ;
+
+		t.engineDefaultDownloadOptions( this->defaultEngineName(),[ this ]( const QString& e ){
+
+			for( int i = 1 ; i < m_table.rowCount() ; i++ ){
+
+				auto u = tableWidget::type::DownloadExtendedOptions ;
+
+				m_table.setDownloadingOptions( u,i,e ) ;
+			}
+		} ) ;
+	} ) ;
+
 	m_table.get().setColumnWidth( 0,m_ctx.Settings().thumbnailWidth( settings::tabName::playlist ) ) ;
 
 	m_table.connect( &QTableWidget::currentItemChanged,[ this ]( QTableWidgetItem * c,QTableWidgetItem * p ){
@@ -264,17 +298,32 @@ playlistdownloader::playlistdownloader( Context& ctx ) :
 			}
 		} ) ;
 
+		const auto& engine = utility::resolveEngine( m_table,this->defaultEngine(),m_ctx.Engines(),row ) ;
+
+		ac = m.addAction( tr( "Show Comments" ) ) ;
+		ac->setEnabled( engine.supportShowingComments() ) ;
+
+		connect( ac,&QAction::triggered,[ this,&engine ](){
+
+			auto row = m_table.currentRow() ;
+
+			if( row != -1 ){
+
+				m_ctx.TabManager().batchDownloader().showComments( engine,m_table.url( row ) ) ;
+			}
+		} ) ;
+
 		utility::addDownloadContextMenu( running,finishSuccess,m,row,[ this ]( int row ){
 
 			auto m = m_table.uiText( row ) ;
 
 			return m.startsWith( engines::engine::mediaAlreadInArchiveText() + "\n" ) ;
 
-		},[ this ]( QAction * ac,bool forceDownload,int row ){
+		},[ this,&engine ]( QAction * ac,bool forceDownload,int row ){
 
-			connect( ac,&QAction::triggered,[ this,row,forceDownload ](){
+			connect( ac,&QAction::triggered,[ &engine,this,row,forceDownload ](){
 
-				downloadManager::index indexes( m_table ) ;
+				downloadManager::index indexes( m_table,false,downloadManager::index::tab::playlist ) ;
 
 				auto e = m_table.runningState( row ) ;
 
@@ -284,9 +333,15 @@ playlistdownloader::playlistdownloader( Context& ctx ) :
 
 					if( u.isEmpty() ){
 
-						indexes.add( row,m_ui.lineEditPLUrlOptions->text() ) ;
+						auto m = m_ui.lineEditPLUrlOptions->text() ;
+
+						auto mm = utility::setDownloadOptions( engine,m_table,row,m ) ;
+
+						indexes.add( row,std::move( mm ) ) ;
 					}else{
-						indexes.add( row,u ) ;
+						auto uu = utility::setDownloadOptions( engine,m_table,row,u ) ;
+
+						indexes.add( row,std::move( uu ) ) ;
 					}
 				}
 
@@ -294,9 +349,9 @@ playlistdownloader::playlistdownloader( Context& ctx ) :
 			} ) ;
 		} ) ;
 
-		utility::saveDownloadList( m_ctx,m,m_table ) ;
+		utility::saveDownloadList( m_ctx,m,m_table,true ) ;
 
-		auto mm = m.addMenu( tableWidget::engineName().replace( ":","" ) ) ;
+		auto mm = m.addMenu( utility::stringConstants::engineName().replace( ":","" ) ) ;
 
 		mm->setEnabled( !finishSuccess ) ;
 
@@ -373,9 +428,15 @@ playlistdownloader::playlistdownloader( Context& ctx ) :
 
 	connect( m_ui.pbPLDownload,&QPushButton::clicked,[ this ](){
 
+		m_banner.clear() ;
 		this->download() ;
 	} ) ;
 
+	connect( m_ui.pbPLChangeTableSize,&QPushButton::clicked,[ this ](){
+
+		this->resizeTable( playlistdownloader::size::toggle ) ;
+	} ) ;
+
 	connect( m_ui.pbPLRangeHistory,&QPushButton::clicked,[ this ](){
 
 		utility::showHistory( *m_ui.lineEditPLDownloadRange,
@@ -424,15 +485,17 @@ playlistdownloader::playlistdownloader( Context& ctx ) :
 
 			auto s = ac->objectName() ;
 
+			const auto& engine = this->defaultEngine() ;
+
 			if( s == "Download All Updated" ){
 
 				m_autoDownload = true ;
 
-				this->getList( std::move( entries ) ) ;
+				this->getListing( std::move( entries ),engine ) ;
 
 			}else if( s == "Show All Updated" ){
 
-				this->getList( std::move( entries ) ) ;
+				this->getListing( std::move( entries ),engine ) ;
 
 			}else if( s == "Manage Subscriptions" ){
 
@@ -448,7 +511,7 @@ playlistdownloader::playlistdownloader( Context& ctx ) :
 
 						ss.emplace_back( it.uiName,it.url,it.getListOptions ) ;
 
-						this->getList( std::move( ss ) ) ;
+						this->getListing( std::move( ss ),engine ) ;
 
 						break ;
 					}
@@ -497,22 +560,32 @@ playlistdownloader::playlistdownloader( Context& ctx ) :
 
 			m_autoDownload = false ;
 
-			this->getList( m ) ;
+			std::vector< subscription::entry > ss ;
+
+			for( const auto& it : util::split( m,' ',true ) ){
+
+				ss.emplace_back( it ) ;
+			}
+
+			this->getListing( std::move( ss ),this->defaultEngine() ) ;
 		}
 	} ) ;
 
 	connect( m_ui.pbPLQuit,&QPushButton::clicked,[ this ](){
 
-		m_tabManager.basicDownloader().appQuit() ;
+		m_ctx.mainWindow().quitApp() ;
 	} ) ;
 }
 
 void playlistdownloader::init_done()
 {
+	m_banner.setBanner( tr( "This May Take A Very Long Time" ) ) ;
 }
 
 void playlistdownloader::enableAll()
 {
+	m_ui.pbClearArchiveFile->setEnabled( true ) ;
+	m_ui.cbUseInternalArchiveFile->setEnabled( true ) ;
 	m_ui.pbPlSubscription->setEnabled( true ) ;
 	m_ui.pbPLPasteClipboard->setEnabled( true ) ;
 	m_ui.lineEditPLUrl->setEnabled( true ) ;
@@ -531,10 +604,14 @@ void playlistdownloader::enableAll()
 	m_ui.cbEngineTypePD->setEnabled( true ) ;
 	m_ui.pbPLRangeHistory->setEnabled( true ) ;
 	m_ui.pbPLOptionsHistory->setEnabled( true ) ;
+	m_ui.pbPLDownloadOptions->setEnabled( true ) ;
 }
 
 void playlistdownloader::disableAll()
 {
+	m_ui.pbPLDownloadOptions->setEnabled( false ) ;
+	m_ui.pbClearArchiveFile->setEnabled( false ) ;
+	m_ui.cbUseInternalArchiveFile->setEnabled( false ) ;
 	m_ui.pbPlSubscription->setEnabled( false ) ;
 	m_ui.pbPLOptionsHistory->setEnabled( false ) ;
 	m_ui.pbPLRangeHistory->setEnabled( false ) ;
@@ -605,6 +682,11 @@ void playlistdownloader::tabExited()
 {
 }
 
+void playlistdownloader::exiting()
+{
+	utility::saveDownloadList( m_ctx,m_table,false ) ;
+}
+
 void playlistdownloader::gotEvent( const QByteArray& )
 {
 }
@@ -637,6 +719,10 @@ void playlistdownloader::updateEnginesList( const QStringList& e )
 				     [ this,s ]( const QString& e ){ m_settings.setDefaultEngine( e,s ) ; } ) ;
 }
 
+void playlistdownloader::clipboardData( const QString& )
+{
+}
+
 QString playlistdownloader::defaultEngineName()
 {
 	return m_settings.defaultEngine( settings::tabName::playlist,m_ctx.Engines().defaultEngineName() ) ;
@@ -644,7 +730,9 @@ QString playlistdownloader::defaultEngineName()
 
 const engines::engine& playlistdownloader::defaultEngine()
 {
-	return m_ctx.Engines().defaultEngine( this->defaultEngineName() ) ;
+	auto id = utility::concurrentID() ;
+
+	return m_ctx.Engines().defaultEngine( this->defaultEngineName(),id ) ;
 }
 
 void playlistdownloader::download()
@@ -663,6 +751,10 @@ void playlistdownloader::download( const engines::engine& engine,downloadManager
 		return ;
 	}
 
+	m_ctx.logger().clear() ;
+
+	m_ctx.logger().setMaxProcessLog( m_table.rowCount() ) ;
+
 	m_ctx.TabManager().basicDownloader().hideTableList() ;
 
 	m_ctx.mainWindow().setTitle( QString() ) ;
@@ -679,7 +771,7 @@ void playlistdownloader::download( const engines::engine& engine,downloadManager
 
 void playlistdownloader::download( const engines::engine& engine )
 {
-	downloadManager::index indexes( m_table ) ;
+	downloadManager::index indexes( m_table,true,downloadManager::index::tab::playlist ) ;
 
 	auto _add = [ & ]( int s,const QString& opts ){
 
@@ -695,9 +787,13 @@ void playlistdownloader::download( const engines::engine& engine )
 
 				if( u.isEmpty() ){
 
-					indexes.add( s,opts ) ;
+					auto oo = utility::setDownloadOptions( engine,m_table,s,opts ) ;
+
+					indexes.add( s,std::move( oo ) ) ;
 				}else{
-					indexes.add( s,u ) ;
+					auto uu = utility::setDownloadOptions( engine,m_table,s,u ) ;
+
+					indexes.add( s,std::move( uu ) ) ;
 				}
 			}
 		}
@@ -723,21 +819,17 @@ void playlistdownloader::download( const engines::engine& eng,int index )
 
 		auto aa = [ this ]( const engines::engine& engine,int index ){
 
+			m_banner.updateTimer() ;
+
 			this->download( engine,index ) ;
 		} ;
 
-		auto bb = [ &engine,index,this ]( const downloadManager::finishedStatus& f ){
+		auto bb = [ &engine,this ]( const downloadManager::finishedStatus& f ){
 
-			utility::updateFinishedState( engine,m_settings,m_table,f ) ;
-
-			if( m_table.noneAreRunning() ){
-
-				m_ctx.TabManager().enableAll() ;
-
-				m_showTimer = false ;
-			}
-
-			m_ctx.mainWindow().setTitle( m_table.completeProgress( index ) ) ;
+			QMetaObject::invokeMethod( this,
+						   "reportFinishedStatus",
+						   Qt::QueuedConnection,
+						   Q_ARG( reportFinished,reportFinished( engine,f ) ) ) ;
 		} ;
 
 		m_ccmd.monitorForFinished( engine,index,std::move( e ),std::move( aa ),std::move( bb ) ) ;
@@ -749,46 +841,87 @@ void playlistdownloader::download( const engines::engine& eng,int index )
 
 	m_settings.addOptionsHistory( m,settings::tabName::playlist ) ;
 
-	auto updater = [ this,index ]( const QString& e ){
+	auto updater = [ this,index ]( const QByteArray& e ){
 
-		m_table.setUiText( e,index ) ;
+		QMetaObject::invokeMethod( this,
+					   "addTextToUi",
+					   Qt::QueuedConnection,
+					   Q_ARG( QByteArray,e ),
+					   Q_ARG( int,index ) ) ;
 	} ;
 
-	auto oopts  = playlistdownloader::make_options( { m_ctx,m_ctx.debug(),false,index },std::move( functions ) ) ;
-	auto logger = make_loggerBatchDownloader( engine.filter( utility::args( m ).quality() ),
+	auto error = []( const QByteArray& ){} ;
+
+	auto id = utility::concurrentID() ;
+	auto oopts  = playlistdownloader::make_options( engine,{ m_ctx,m_ctx.debug(),false,index },std::move( functions ) ) ;
+	auto logger = make_loggerBatchDownloader( engine.filter( id,utility::args( m,engine ).quality() ),
 						  m_ctx.logger(),
 						  std::move( updater ),
-						  utility::concurrentID() ) ;
+						  std::move( error ),
+						  id ) ;
 
 	m_table.setRunningState( downloadManager::finishedStatus::running(),index ) ;
 
 	auto optsUpdater = [ this ]( QStringList opts ){
 
-		const auto& archivePath = m_subscription.archivePath() ;
-
-		if( !opts.contains( "--download-archive" ) && !archivePath.isEmpty() ){
+		if( m_ctx.Settings().useInternalArchiveFile() ){
 
 			opts.append( "--download-archive" ) ;
-			opts.append( archivePath ) ;
+			opts.append( m_subscription.archivePath() ) ;
 		}
 
 		return opts ;
 	} ;
 
 	m_ccmd.download( engine,
-			 optsUpdater,
-			 m_ctx.Engines().engineDirPaths(),
+			 std::move( optsUpdater ),
 			 m_table.url( index ),
+			 m_ctx,
 			 m_terminator.setUp(),
 			 std::move( oopts ),
 			 std::move( logger ) ) ;
 }
 
-void playlistdownloader::getList( playlistdownloader::listIterator iter )
+void playlistdownloader::showBanner()
+{
+	m_table.clear() ;
+	m_banner.clear() ;
+
+	QIcon icon( ":/media-downloader" ) ;
+
+	auto w = m_settings.thumbnailWidth( settings::tabName::playlist ) ;
+	auto h = m_settings.thumbnailHeight( settings::tabName::playlist ) ;
+
+	tableWidget::entry entry ;
+
+	entry.uiText    = m_banner.txt() ;
+	entry.thumbnail = icon.pixmap( w,h ) ;
+
+	m_table.addItem( std::move( entry ) ) ;
+	m_table.selectLast() ;
+
+	m_ctx.TabManager().disableAll() ;
+}
+
+void playlistdownloader::getListing( playlistdownloader::listIterator e,const engines::engine& engine )
+{
+	this->resizeTable( playlistdownloader::size::large ) ;
+
+	this->showBanner() ;
+
+	engine.updateVersionInfo( m_ctx,[ this,&engine,e = std::move( e ) ](){
+
+		this->getList( e.move(),engine ) ;
+	} ) ;
+}
+
+void playlistdownloader::getList( playlistdownloader::listIterator iter,
+				  const engines::engine& engine )
 {
 	m_dataReceived = false ;
 	m_stoppedOnExisting = false ;
-	m_meaw = false ;
+	m_stillProcessingJsonOutput = false ;
+	m_downloaderId++ ;
 
 	auto url = iter.url() ;
 
@@ -798,52 +931,49 @@ void playlistdownloader::getList( playlistdownloader::listIterator iter )
 
 	m_ui.pbPLCancel->setEnabled( true ) ;
 
-	const auto& engine = this->defaultEngine() ;
-
 	auto opts = engine.dumpJsonArguments() ;
 
-	auto listOpts = iter.listOptions() ;
+	if( !opts.isEmpty() ){
 
-	if( listOpts.isEmpty() ){
-
-		listOpts = m_ui.lineEditPLDownloadRange->text() ;
-
-		if( !listOpts.isEmpty() ){
-
-			m_settings.addToplaylistRangeHistory( listOpts ) ;
-
-			m_settings.setPlaylistRangeHistoryLastUsed( listOpts ) ;
-		}
+		opts.last() += m_jsonEndMarker ;
 	}
 
-	if( !listOpts.isEmpty() ){
+	auto configListOpts = iter.listOptions() ;
+	auto listOptions    = m_ui.lineEditPLDownloadRange->text() ;
 
-		if( listOpts.startsWith( "--" ) ){
+	if( !listOptions.isEmpty() ){
 
-			opts.append( util::split( listOpts,' ',true ) ) ;
-		}else{
-			opts.append( engine.playlistItemsArgument() ) ;
+		m_settings.addToplaylistRangeHistory( listOptions ) ;
 
-			auto m = util::split( listOpts,' ',true ) ;
+		m_settings.setPlaylistRangeHistoryLastUsed( listOptions ) ;
 
-			opts.append( m ) ;
-		}
+		opts.append( util::split( listOptions,' ',true ) ) ;
 	}
 
+	if( !configListOpts.isEmpty() ){
+
+		opts.append( util::split( configListOpts,' ',true ) ) ;
+	}
+
+	engine.setTextEncondig( opts ) ;
+	engine.updateGetPlaylistCmdOptions( opts ) ;
+
+	opts.append( "-v" ) ;
+
 	opts.append( url ) ;
 
 	m_networkRunning = 0 ;
 
-	util::runInBgThread( [ &engine,this,opts = std::move( opts ) ]()mutable{
+	utils::qthread::run( [ &engine,this,opts = std::move( opts ) ]()mutable{
 
 		return customOptions( std::move( opts ),
 				      m_subscription.archivePath(),
-				      m_settings,
-				      engine ) ;
+				      engine,
+				      m_ctx ) ;
 
-	},[ this,&engine,iter = std::move( iter ) ]( customOptions&& c )mutable{
+	},[ this,&engine,iter = std::move( iter ) ]( customOptions&& c ){
 
-		this->getList( std::move( c ),engine,std::move( iter ) ) ;
+		this->getList( std::move( c ),engine,iter.move() ) ;
 	} ) ;
 }
 
@@ -857,19 +987,18 @@ void playlistdownloader::getList( customOptions&& c,
 			m_gettingPlaylist = true ;
 			m_ui.pbPLCancel->setEnabled( true ) ;
 
-		},[ this,iter = std::move( iter ) ]( utility::ProcessExitState st,const playlistdownloader::opts& ){
+		},[ &engine,this,iter = std::move( iter ) ]( utility::ProcessExitState st,const playlistdownloader::opts& ){
 
-			if( m_meaw ){
+			if( m_stillProcessingJsonOutput ){
 
 				if( iter.hasNext() ){
 
-					this->getList( iter.next() ) ;
+					this->getList( iter.next(),engine ) ;
 				}else{
 					if( m_autoDownload ){
 
 						this->download() ;
 					}else{
-						m_showTimer = false ;
 						m_ctx.TabManager().enableAll() ;
 						m_gettingPlaylist = false ;
 						m_ui.pbPLCancel->setEnabled( false ) ;
@@ -880,9 +1009,8 @@ void playlistdownloader::getList( customOptions&& c,
 
 				if( m_stoppedOnExisting && iter.hasNext() ){
 
-					this->getList( iter.next() ) ;
+					this->getList( iter.next(),engine ) ;
 				}else{
-					m_showTimer = false ;
 					m_ctx.TabManager().enableAll() ;
 					m_gettingPlaylist = false ;
 					m_ui.pbPLCancel->setEnabled( false ) ;
@@ -890,73 +1018,131 @@ void playlistdownloader::getList( customOptions&& c,
 
 			}else if( iter.hasNext() ){
 
-				this->getList( iter.next() ) ;
+				this->getList( iter.next(),engine ) ;
 			}else{
 				m_ctx.TabManager().enableAll() ;
 				m_gettingPlaylist = false ;
 				m_ui.pbPLCancel->setEnabled( false ) ;
-
-				if( !m_dataReceived ){
-
-					m_showTimer = false ;
-				}
 			}
 		}
 	) ;
 
 	auto opts = c.options() ;
 
-	auto bb = [ copts = std::move( c ),this ]( tableWidget& table,Logger::Data& data ){
+	auto stdOut = [ copts = std::move( c ),this ]( tableWidget& table,Logger::Data& data ){
 
 		m_dataReceived = true ;
 
-		while( true ){
+		int position = 0 ;
 
-			if( this->parseJson( copts,table,data ) == Loop::Break ){
+		const auto line = data.toLine() ;
 
-				break ;
+		if( line.contains( m_jsonEndMarker ) ){
+
+			while( true ){
+
+				auto m = line.indexOf( m_jsonEndMarker,position ) ;
+
+				if( m == -1 ){
+
+					break ;
+				}else{
+					utility::MediaEntry media( line.mid( position,m ) ) ;
+
+					if( media.valid() ){
+
+						if( this->parseJson( copts,table,std::move( media ) ) ){
+
+							break ;
+						}
+					}
+
+					position = position + m + m_jsonEndMarker.size() ;
+				}
 			}
+
+			data.clear() ;
+			data.add( line.mid( position ),utility::concurrentID() ) ;
 		}
 	} ;
 
+	auto stdError = [ this ]( const QByteArray& e ){
+
+		auto s = e.indexOf( "page" ) ;
+
+		if( s != -1 && e.contains( "Downloading API JSON" ) ){
+
+			auto m = e.mid( s ) ;
+
+			s = m.indexOf( ':' ) ;
+
+			if( s != -1 ){
+
+				m = m.mid( 0,s ).mid( 4 ) ;
+
+				m_banner.updateProgress( tr( "Number of Pages Downloaded" ) + ": " + m ) ;
+			}
+
+			return false ;
+
+		}else if( utils::misc::startsWithAny( e,"[download] Downloading video ","[download] Downloading item " ) ){
+
+			auto s = [ & ](){
+
+				if( e.startsWith( "[download] Downloading video " ) ){
+
+					return 29 ;
+				}else{
+					return 28 ;
+				}
+			}() ;
+
+			auto m = tr( "Downloading video info" ) + " " + e.mid( s ) ;
+
+			s = m.indexOf( '\n' ) ;
+
+			if( s != -1 ){
+
+				m = m.mid( 0,s ) ;
+
+				m.replace( " of N/A","" ) ;
+			}
+
+			m_banner.updateProgress( m ) ;
+
+			return false ;
+		}
+
+		if( e.startsWith( "ERROR: " ) ){
+
+			if( e.contains( "Temporary failure in name resolution" ) ){
+
+				m_banner.reportError( "ERROR\nNetwork Not Reachable" ) ;
+			}else{
+				auto m = util::split( e,'\n',true ).at( 0 ) ;
+				m_banner.reportError( "ERROR\n" + m.mid( 7 ) ) ;
+			}
+
+			return true ;
+		}
+
+		return e.startsWith( "WARNING" ) ;
+	} ;
+
 	auto id     = utility::concurrentID() ;
-	auto oopts  = playlistdownloader::make_options( { m_ctx,m_ctx.debug(),false,-1 },std::move( functions ) ) ;
-	auto logger = make_loggerPlaylistDownloader( m_table,m_ctx.logger(),id,std::move( bb ) ) ;
+	auto oopts  = playlistdownloader::make_options( engine,{ m_ctx,m_ctx.debug(),false,-1 },std::move( functions ) ) ;
+	auto logger = make_loggerPlaylistDownloader( m_table,m_ctx.logger(),id,std::move( stdOut ),std::move( stdError ) ) ;
 	auto term   = m_terminator.setUp( m_ui.pbPLCancel,&QPushButton::clicked,-1 ) ;
 	auto ch     = QProcess::ProcessChannel::StandardOutput ;
-	auto argsq  = utility::args( m_ui.lineEditPLUrlOptions->text() ).quality() ;
+	auto argsq  = utility::args( m_ui.lineEditPLUrlOptions->text(),engine ).quality() ;
 
 	if( !m_gettingPlaylist ){
 
 		logger.clear() ;
 
-		auto s = tr( "This May Take A Very Long Time" ) ;
-		auto d = engines::engine::functions::timer::stringElapsedTime( 0 ) ;
+		this->showBanner() ;
 
-		QIcon icon( ":/media-downloader" ) ;
-
-		auto w = m_settings.thumbnailWidth( settings::tabName::playlist ) ;
-		auto h = m_settings.thumbnailHeight( settings::tabName::playlist ) ;
-
-		m_table.addItem( { icon.pixmap( w,h ),d + "\n" + s,"","" } ) ;
-
-		m_showTimer = true ;
-
-		util::Timer( 1000,[ this,s ]( int counter ){
-
-			if( m_showTimer ){
-
-				using tt = engines::engine::functions ;
-
-				auto duration = tt::timer::stringElapsedTime( counter * 1000 ) ;
-
-				m_table.setUiText( duration + "\n" + s,0 ) ;
-
-				return false ;
-			}else{
-				return true ;
-			}
-		} ) ;
+		m_ctx.logger().setMaxProcessLog( m_table.rowCount() ) ;
 	}
 
 	m_table.selectLast() ;
@@ -968,7 +1154,6 @@ void playlistdownloader::getList( customOptions&& c,
 				      ch ) ;
 
 	utility::run( opts,argsq,std::move( ctx ) ) ;
-
 }
 
 void playlistdownloader::clearScreen()
@@ -985,76 +1170,11 @@ bool playlistdownloader::enabled()
 	return m_ui.lineEditPLUrl->isEnabled() ;
 }
 
-playlistdownloader::Loop playlistdownloader::parseJson( const customOptions& copts,
-							tableWidget& table,
-							Logger::Data& data )
+bool playlistdownloader::parseJson( const customOptions& copts,
+				    tableWidget& table,
+				    utility::MediaEntry media )
 {
-	auto mmm = data.toLine() ;
-
-	auto oo = mmm.indexOf( '{' ) ;
-
-	if( oo == -1 ){
-
-		return Loop::Break ;
-	}
-
-	int counter = 0 ;
-	int index = 0 ;
-
-	while( true ){
-
-		if( index >= mmm.size() ){
-
-			return Loop::Break ;
-		}
-
-		auto a = mmm[ index ] ;
-
-		if( a == '{' ){
-
-			counter++ ;
-
-		}else if( a == '}' ){
-
-			counter-- ;
-
-			if( counter == 0 ){
-
-				break ;
-			}
-		}
-
-		index++ ;
-	}
-
-	m_meaw = true ;
-
-	utility::MediaEntry media( "",mmm.mid( oo,index + 1 ) ) ;
-
-	if( !media.valid() ){
-
-		return Loop::Break ;
-	}else{
-		data.clear() ;
-		data.add( mmm.mid( index + 1 ) ) ;
-	}	
-
-	auto _show = [ this,&table ]( const tableWidget::entry& e )
-	{
-		auto row = table.addItem( e ) ;
-
-		m_ctx.TabManager().Configure().setDownloadOptions( row,table ) ;
-
-		if( !m_ui.pbPLCancel->isEnabled() ){
-
-			if( m_autoDownload ){
-
-				this->download() ;
-			}else{
-				m_showTimer = false ;
-			}
-		}
-	} ;
+	m_stillProcessingJsonOutput = true ;
 
 	if( copts.contains( media.id() ) ){
 
@@ -1062,19 +1182,24 @@ playlistdownloader::Loop playlistdownloader::parseJson( const customOptions& cop
 
 			m_stoppedOnExisting = true ;
 			m_ui.pbPLCancel->click() ;
-			return Loop::Break ;
+
+			return true ;
 
 		}else if( copts.skipOnExisting() ){
 
 			auto s = downloadManager::finishedStatus::finishedWithSuccess() ;
 
-			auto a = QObject::tr( "Media Already In Archive" ) + "\n" + media.uiText() ;
+			const auto& img = m_defaultVideoThumbnailIcon ;
 
-			_show( { m_defaultVideoThumbnailIcon,a,media.url(),s } ) ;
+			tableWidget::entry entry{ img,s,media } ;
+
+			entry.uiText = QObject::tr( "Media Already In Archive" ) + "\n" + media.uiText() ;
+
+			this->showEntry( table,std::move( entry ) ) ;
 
 			table.selectLast() ;
 
-			return Loop::Continue ;
+			return false ;
 		}
 	}
 
@@ -1082,64 +1207,172 @@ playlistdownloader::Loop playlistdownloader::parseJson( const customOptions& cop
 
 	if( max > 0 && media.intDuration() > max ){
 
-		return Loop::Continue ;
+		return false ;
 	}
 
 	auto min = copts.minMediaLength() ;
 
 	if( min > 0 && media.intDuration() < min ){
 
-		return Loop::Continue ;
+		return false ;
 	}
 
-	auto s = downloadManager::finishedStatus::notStarted() ;
-
 	table.selectLast() ;
 
 	if( networkAccess::hasNetworkSupport() ){
 
-		m_meaw = false ;
+		m_stillProcessingJsonOutput = false ;
 
-		auto& network = m_ctx.versionInfo().network() ;
+		auto& network = m_ctx.network() ;
 
 		m_networkRunning++ ;
 
 		auto thumbnailUrl = media.thumbnailUrl() ;
 
-		network.getResource( thumbnailUrl,
-				     [ this,&table,s,_show = std::move( _show ),
-				     media = std::move( media ) ]( const QByteArray& data ){
+		network.get( thumbnailUrl,
+			     [ this,&table,id = m_downloaderId,
+			     media = media.move() ]( const utils::network::reply& reply ){
 
-			QPixmap pixmap ;
-
-			if( pixmap.loadFromData( data ) ){
-
-				auto width = m_settings.thumbnailWidth( settings::tabName::playlist ) ;
-				auto height = m_settings.thumbnailHeight( settings::tabName::playlist ) ;
-
-				_show( { pixmap.scaled( width,height ),media.uiText(),media.url(),s } ) ;
-			}else{
-				_show( { m_defaultVideoThumbnailIcon,media.uiText(),media.url(),s } ) ;
-			}
-
-			table.selectLast() ;
-
-			m_networkRunning-- ;
+			utility::networkReply( this,"networkData",m_ctx,reply,&table,id,media.move() ) ;
 		} ) ;
 	}else{
-		_show( { m_defaultVideoThumbnailIcon,media.uiText(),media.url(),s } ) ;
-
-		table.selectLast() ;
+		utility::networkReply( this,"networkData",&table,m_downloaderId,media.move() ) ;
 	}
 
-	return Loop::Continue ;
+	return false ;
+}
+
+void playlistdownloader::networkData( utility::networkReply m )
+{
+	if( m.id() < m_downloaderId ){
+
+		/*
+		 * Network responce took too long and we are now running
+		 * a newer request to get playlist entries
+		 */
+		return ;
+	}
+
+	auto& table = m.table() ;
+	auto& media = m.media() ;
+
+	auto s = downloadManager::finishedStatus::notStarted() ;
+
+	if( networkAccess::hasNetworkSupport() ){
+
+		QPixmap pixmap ;
+
+		if( m.success() && pixmap.loadFromData( m.data() ) ){
+
+			auto width = m_settings.thumbnailWidth( settings::tabName::playlist ) ;
+			auto height = m_settings.thumbnailHeight( settings::tabName::playlist ) ;
+
+			auto img = pixmap.scaled( width,height ) ;
+
+			this->showEntry( table,{ img,s,media } ) ;
+		}else{
+			const auto& img = m_defaultVideoThumbnailIcon ;
+
+			this->showEntry( table,{ img,s,media } ) ;
+		}
+	}else{
+		const auto& img = m_defaultVideoThumbnailIcon ;
+
+		this->showEntry( table,{ img,s,media } ) ;
+	}
+
+	table.selectLast() ;
+
+	m_networkRunning-- ;
+}
+
+void playlistdownloader::addTextToUi( const QByteArray& data,int index )
+{
+	if( downloadManager::finishedStatus::running( m_table.runningState( index ) ) ){
+
+		m_banner.updateTimer() ;
+
+		m_table.setUiText( data,index ) ;
+	}
+}
+
+void playlistdownloader::reportFinishedStatus( const reportFinished& f )
+{
+	m_banner.updateTimer() ;
+
+	utility::updateFinishedState( f.engine(),m_settings,m_table,f.finishedStatus() ) ;
+
+	if( m_table.noneAreRunning() ){
+
+		m_ctx.TabManager().enableAll() ;
+
+		if( m_table.allFinishedWithSuccess() ){
+
+			this->resizeTable( playlistdownloader::size::small ) ;
+		}
+	}
+
+	m_ctx.mainWindow().setTitle( m_table.completeProgress( 1 ) ) ;
+}
+
+void playlistdownloader::resizeTable( playlistdownloader::size s )
+{
+	bool e = [ & ](){
+
+		if( s == playlistdownloader::size::toggle ){
+
+			return m_ui.pbPLPasteClipboard->isVisible() ;
+		}else{
+			return s == playlistdownloader::size::large ;
+		}
+	}() ;
+
+	m_ui.pbPLPasteClipboard->setVisible( !e ) ;
+	m_ui.pbPlSubscription->setVisible( !e ) ;
+	m_ui.pbPLRangeHistory->setVisible( !e ) ;
+	m_ui.pbClearArchiveFile->setVisible( !e ) ;
+	m_ui.cbUseInternalArchiveFile->setVisible( !e ) ;
+	m_ui.pbPLOptionsHistory->setVisible( !e ) ;
+	m_ui.pbPLDownloadOptions->setVisible( !e ) ;
+	m_ui.cbEngineTypePD->setVisible( !e ) ;
+	m_ui.labelPLEngineName->setVisible( !e ) ;
+
+	if( e ){
+
+		m_ui.pbPLChangeTableSize->setToolTip( tr( "Show Controls" ) ) ;
+		m_ui.pbPLChangeTableSize->setText( tr( "Show" ) ) ;
+
+		m_ui.tableWidgetPl->resize( 771,431 ) ;
+	}else{
+		m_ui.pbPLChangeTableSize->setToolTip( tr( "Hide Controls" ) ) ;
+		m_ui.pbPLChangeTableSize->setText( tr( "Hide" ) ) ;
+
+		m_ui.tableWidgetPl->resize( 771,271 ) ;
+	}
+}
+
+void playlistdownloader::showEntry( tableWidget& table,tableWidget::entry e )
+{
+	auto row = table.addItem( std::move( e ) ) ;
+
+	m_ctx.logger().setMaxProcessLog( m_table.rowCount() ) ;
+
+	m_ctx.TabManager().Configure().setDownloadOptions( row,table ) ;
+
+	if( !m_ui.pbPLCancel->isEnabled() ){
+
+		if( m_autoDownload ){
+
+			this->download() ;
+		}
+	}
 }
 
 playlistdownloader::subscription::subscription( const Context& e,
 						tableMiniWidget< int >& t,
 						QWidget& w ) :
 	m_path( e.Engines().engineDirPaths().dataPath( "subscriptions.json" ) ),
-	m_archivePath( e.Engines().engineDirPaths().dataPath( "subscriptions_archive_file.txt" ) ),
+	m_archivePath( e.Engines().engineDirPaths().subscriptionsArchiveFilePath() ),
 	m_table( t ),
 	m_ui( w )
 {
@@ -1264,3 +1497,23 @@ void playlistdownloader::subscription::save()
 
 	f.write( QJsonDocument( m_array ).toJson( QJsonDocument::Indented ) ) ;
 }
+
+void playlistdownloader::banner::updateProgress( const QString& progress )
+{
+	m_time = m_timer.elapsedTime() ;
+	auto duration = engines::engine::functions::timer::stringElapsedTime( m_time ) ;
+	m_progress = duration + ", " + progress ;
+	m_table.setUiText( m_txt + "\n" + m_progress,0 ) ;
+}
+
+void playlistdownloader::banner::reportError( const QString& e )
+{
+	m_table.setUiText( e,0 ) ;
+}
+
+void playlistdownloader::banner::updateTimer()
+{
+	auto m = m_timer.elapsedTime() ;
+	auto duration = engines::engine::functions::timer::stringElapsedTime( m_time + m ) ;
+	m_table.setUiText( m_txt + "\n" + duration,0 ) ;
+}
