@@ -22,11 +22,12 @@
 #include "locale_path.h"
 #include "translator.h"
 #include "logger.h"
+#include "themes.h"
 
 #include <QDir>
-#include <QApplication>
 #include <QFile>
-#include <QStyleFactory>
+
+#include <cstring>
 
 static QString _configPath()
 {
@@ -37,7 +38,7 @@ static QString _configPath()
 
 		return QDir::homePath() + "/.config/media-downloader/" ;
 	}else{
-		return s.first() ;
+		return s.first() + "/media-downloader/" ;
 	}
 #else
 	return QDir::homePath() + "/.config/media-downloader/" ;
@@ -46,30 +47,62 @@ static QString _configPath()
 
 bool settings::portableVersion()
 {
-	if( utility::platformIsWindows() ){
+	return m_portableVersion ;
+}
 
-		auto a = QFile::exists( settings::portableVersionConfigPath() ) ;
-		auto b = QFile::exists( QDir::currentPath() + "/media-downloader.exe" ) ;
+const QString& settings::runningUpdatedText()
+{
+	return m_runningUpdated ;
+}
 
-		return a && b ;
+static QString _monitorClipboadUrl( settings::tabName e )
+{
+	if( e == settings::tabName::basic ){
+
+		return "BasicDownloaderMonitorClipboadUrl" ;
+
+	}else if( e == settings::tabName::batch ){
+
+		return "BatchDownloaderMonitorClipboadUrl" ;
+
+	}else if( e == settings::tabName::playlist ){
+
+		return "PlaylistDownloaderMonitorClipboadUrl" ;
 	}else{
-		return false ;
+		return "" ;
 	}
 }
 
-QString settings::portableVersionConfigPath()
+void settings::setMonitorClipboardUrl( bool e,settings::tabName t )
 {
-	return QDir::currentPath() + "/local" ;
+	m_settings.setValue( _monitorClipboadUrl( t ),e ) ;
 }
 
-QString settings::darkMode()
+bool settings::monitorClipboardUrl( settings::tabName tabName )
 {
-	if( !m_settings.contains( "DarkModeName" ) ){
+	auto m = _monitorClipboadUrl( tabName ) ;
 
-		m_settings.setValue( "DarkModeName",settings::darkModes().unTranslatedAt( 0 ) ) ;
+	if( !m_settings.contains( m ) ){
+
+		m_settings.setValue( m,false ) ;
 	}
 
-	return m_settings.value( "DarkModeName" ).toString() ;
+	return m_settings.value( m ).toBool() ;
+}
+
+QString settings::themeName()
+{
+	if( !m_settings.contains( "ThemeName" ) ){
+
+		m_settings.setValue( "ThemeName",themes().unTranslatedAt( 0 ) ) ;
+	}
+
+	return m_settings.value( "ThemeName" ).toString() ;
+}
+
+void settings::setThemeName( const QString& e )
+{
+	m_settings.setValue( "ThemeName",e ) ;
 }
 
 static QString _getOptionsHistoryTabName( settings::tabName e )
@@ -160,6 +193,112 @@ void settings::clearPlaylistUrlHistory()
 	m_settings.setValue( "PlaylistUrlHistory",QStringList() ) ;
 }
 
+void settings::setAutoSavePlaylistOnExit( bool e )
+{
+	m_settings.setValue( "AutoSavePlaylistOnExit",e ) ;
+}
+
+bool settings::autoSavePlaylistOnExit()
+{
+	if( !m_settings.contains( "AutoSavePlaylistOnExit" ) ){
+
+		m_settings.setValue( "AutoSavePlaylistOnExit",true ) ;
+	}
+
+	return m_settings.value( "AutoSavePlaylistOnExit" ).toBool() ;
+}
+
+bool settings::useInternalArchiveFile()
+{
+	if( !m_settings.contains( "UseInternalArchiveFile" ) ){
+
+		m_settings.setValue( "UseInternalArchiveFile",true ) ;
+	}
+
+	return m_settings.value( "UseInternalArchiveFile" ).toBool() ;
+}
+
+bool settings::checkForUpdates()
+{
+	if( !m_settings.contains( "CheckForUpdates" ) ){
+
+		m_settings.setValue( "CheckForUpdates",utility::platformIsWindows() ) ;
+	}
+
+	return m_settings.value( "CheckForUpdates" ).toBool() ;
+}
+
+bool settings::enableLibraryTab()
+{
+	if( !m_settings.contains( "EnableLibraryTab" ) ){
+
+		m_settings.setValue( "EnableLibraryTab",false ) ;
+	}
+
+	return m_settings.value( "EnableLibraryTab" ).toBool() ;
+}
+
+bool settings::checkForEnginesUpdates()
+{
+	if( !m_settings.contains( "CheckForEnginesUpdates" ) ){
+
+		m_settings.setValue( "CheckForEnginesUpdates",true ) ;
+	}
+
+	return m_settings.value( "CheckForEnginesUpdates" ).toBool() ;
+}
+
+int settings::textAlignment()
+{
+	if( !m_settings.contains( "MainTableTextAlignment" ) ){
+
+		m_settings.setValue( "MainTableTextAlignment","center" ) ;
+	}
+
+	auto m = m_settings.value( "MainTableTextAlignment" ).toString() ;
+
+	if( m == "center" ){
+
+		return Qt::AlignCenter ;
+
+	}else if( m == "left" ){
+
+		return Qt::AlignLeft | Qt::AlignVCenter ;
+
+	}else if( m == "right" ){
+
+		return Qt::AlignRight | Qt::AlignVCenter;
+	}else{
+		m_settings.setValue( "MainTableTextAlignment","center" ) ;
+		return Qt::AlignCenter ;
+	}
+}
+
+void settings::setEnableLibraryTab( bool e )
+{
+	m_settings.setValue( "EnableLibraryTab",e ) ;
+}
+
+void settings::setCheckForUpdates( bool e )
+{
+	m_settings.setValue( "CheckForUpdates",e ) ;
+}
+
+void settings::setUseInternalArchiveFile( bool e )
+{
+	m_settings.setValue( "UseInternalArchiveFile",e ) ;
+}
+
+int settings::networkTimeOut()
+{
+	if( !m_settings.contains( "NetworkTimeOutInSeconds" ) ){
+
+		m_settings.setValue( "NetworkTimeOutInSeconds",30 ) ;
+	}
+
+	return m_settings.value( "NetworkTimeOutInSeconds" ).toInt() * 1000 ;
+}
+
 QStringList settings::playlistRangeHistory()
 {
 	if( !m_settings.contains( "PlaylistRangeHistory" ) ){
@@ -180,11 +319,6 @@ QStringList settings::playlistUrlHistory()
 	return m_settings.value( "PlaylistUrlHistory" ).toStringList() ;
 }
 
-void settings::setDarkMode( const QString& e )
-{
-	m_settings.setValue( "DarkModeName",e ) ;
-}
-
 void settings::setPlaylistRangeHistoryLastUsed( const QString& e )
 {
 	m_settings.setValue( "playlistRangeHistoryLastUsed",e ) ;
@@ -209,49 +343,92 @@ static std::unique_ptr< QSettings > _set_config( const QString& path )
 	return std::make_unique< QSettings >( m,QSettings::IniFormat ) ;
 }
 
-static std::unique_ptr< QSettings > _init()
+static bool _portableVersionInit( const utility::cliArguments& args,
+				  QString& dataPath,
+				  QString& exePath,
+				  QString& exeOrgPath )
 {
-	if( settings::portableVersion() ){
+	if( utility::platformIsWindows() ){
 
-		return _set_config( settings::portableVersionConfigPath() ) ;
-	}else{
-		if( utility::platformIsWindows() ){
+		exePath = utility::windowsApplicationDirPath() ;
 
-			auto appPath      = _configPath() + "/media-downloader" ;
-			auto settingsPath = appPath + "/settings" ;
+		exeOrgPath = args.originalPath() ;
 
-			if( QFile::exists( settingsPath ) ){
+		if( exeOrgPath.isEmpty() ){
 
-				return _set_config( appPath ) ;
-			}else{
-				/*
-				 * Migrating from registry based config to text file config.
-				 */
-				QSettings oldSettings( "media-downloader","media-downloader" ) ;
-
-				auto newSettings = _set_config( appPath ) ;
-
-				const auto keys = oldSettings.allKeys() ;
-
-				for( const auto& it : keys ){
-
-					newSettings->setValue( it,oldSettings.value( it ) ) ;
-				}
-
-				oldSettings.clear() ;
-
-				return newSettings ;
-			}
-		}else{
-			return std::make_unique< QSettings >( "media-downloader","media-downloader" ) ;
+			exeOrgPath = exePath ;
 		}
+
+		if( args.portable() ){
+
+			dataPath = args.dataPath() ;
+
+			return true ;
+		}else{
+			auto a = exePath  + "/local" ;
+
+			if( QFile::exists( a ) ){
+
+				dataPath = a ;
+
+				return true ;
+			}else{
+				dataPath = _configPath() ;
+
+				return false ;
+			}
+		}
+	}else{
+		dataPath = _configPath() ;
+
+		return false ;
 	}
 }
 
-settings::settings() :
-	m_settingsP( _init() ),
-	m_settings( *m_settingsP ),
-	m_portableVersion( settings::portableVersion() )
+static std::unique_ptr< QSettings > _init( const QString& dataPath,bool portableVersion )
+{	
+	if( utility::platformIsWindows() ){
+
+		if( portableVersion ){
+
+			return _set_config( dataPath ) ;
+		}
+
+		auto appPath      = _configPath() ;
+		auto settingsPath = appPath + "/settings" ;
+
+		if( QFile::exists( settingsPath ) ){
+
+			return _set_config( appPath ) ;
+		}else{
+			/*
+			 * Migrating from registry based config to text file config.
+			 */
+			QSettings oldSettings( "media-downloader","media-downloader" ) ;
+
+			auto newSettings = _set_config( appPath ) ;
+
+			const auto keys = oldSettings.allKeys() ;
+
+			for( const auto& it : keys ){
+
+				newSettings->setValue( it,oldSettings.value( it ) ) ;
+			}
+
+			oldSettings.clear() ;
+
+			return newSettings ;
+		}
+	}else{
+		return std::make_unique< QSettings >( "media-downloader","media-downloader" ) ;
+	}
+}
+
+settings::settings( const utility::cliArguments& args ) :
+	m_portableVersion( _portableVersionInit( args,m_dataPath,m_exePath,m_exeOrgPath ) ),
+	m_settingsP( _init( m_dataPath,portableVersion() ) ),
+	m_settings( *m_settingsP )
+
 {
 #if QT_VERSION >= QT_VERSION_CHECK( 5,6,0 )
 
@@ -259,13 +436,21 @@ settings::settings() :
 	QApplication::setAttribute( Qt::AA_EnableHighDpiScaling ) ;
 #else
 	m_EnableHighDpiScaling = false ;
-#endif	
+#endif
 	auto m = this->highDpiScalingFactor() ;
 
 	if( m != "1.0" ){
 
 		qputenv( "QT_SCALE_FACTOR",m ) ;
 	}
+
+	if( args.runningUpdated() ){
+
+		//const auto& m = args.originalVersion() ;
+		//const auto& mm = utility::runningVersionOfMediaDownloader() ;
+
+		//m_runningUpdated = QObject::tr( "Started As Version %1 And Now Running As Version %2" ).arg( m,mm ) ;
+	}
 }
 
 QSettings& settings::bk()
@@ -288,6 +473,16 @@ int settings::tabNumber()
 	return m_settings.value( "TabNumber" ).toInt() ;
 }
 
+int settings::maxLoggerProcesses()
+{
+	if( !m_settings.contains( "MaxLoggerProcesses" ) ){
+
+		m_settings.setValue( "MaxLoggerProcesses",0 ) ;
+	}
+
+	return m_settings.value( "MaxLoggerProcesses" ).toInt() ;
+}
+
 size_t settings::maxConcurrentDownloads()
 {
 	if( !m_settings.contains( "MaxConcurrentDownloads" ) ){
@@ -298,6 +493,11 @@ size_t settings::maxConcurrentDownloads()
 	return static_cast< size_t >( m_settings.value( "MaxConcurrentDownloads" ).toInt() ) ;
 }
 
+const QString & settings::exeOriginalPath()
+{
+	return m_exeOrgPath ;
+}
+
 void settings::setMaxConcurrentDownloads( int s )
 {
 	m_settings.setValue( "MaxConcurrentDownloads",s ) ;
@@ -380,14 +580,14 @@ static QString _downloadFolder( QSettings& settings,bool portableVersion,Functio
 
 QString settings::downloadFolder()
 {
-	return _downloadFolder( m_settings,m_portableVersion,[]( const QString& ){} ) ;
+	return _downloadFolder( m_settings,this->portableVersion(),[]( const QString& ){} ) ;
 }
 
 QString settings::downloadFolder( Logger& logger )
 {
-	return _downloadFolder( m_settings,m_portableVersion,[ &logger ]( const QString& e ){
+	return _downloadFolder( m_settings,this->portableVersion(),[ &logger ]( const QString& e ){
 
-		logger.add( e ) ;
+		logger.add( e,utility::sequentialID() ) ;
 	} ) ;
 }
 
@@ -401,6 +601,11 @@ bool settings::showTrayIcon()
 	return m_settings.value( "ShowTrayIcon" ).toBool() ;
 }
 
+void settings::setshowTrayIcon( bool e )
+{
+	m_settings.setValue( "ShowTrayIcon",e ) ;
+}
+
 bool settings::autoDownload()
 {
 	if( !m_settings.contains( "AutoDownload" ) ){
@@ -433,11 +638,6 @@ bool settings::concurrentDownloading()
 
 bool settings::useSystemProvidedVersionIfAvailable()
 {
-	if( utility::platformIsWindows() ){
-
-		m_settings.setValue( "UseSystemProvidedVersionIfAvailable",false ) ;
-	}
-
 	if( !m_settings.contains( "UseSystemProvidedVersionIfAvailable" ) ){
 
 		if( utility::platformIsWindows() ){
@@ -451,25 +651,6 @@ bool settings::useSystemProvidedVersionIfAvailable()
 	return m_settings.value( "UseSystemProvidedVersionIfAvailable" ).toBool() ;
 }
 
-QString settings::engineDefaultDownloadOptions( const QString& engineName )
-{
-	auto m = "EngineDefaultDownloadingOptions_" + engineName ;
-
-	if( !m_settings.contains( m ) ){
-
-		m_settings.setValue( m,QString() ) ;
-	}
-
-	return m_settings.value( m ).toString() ;
-}
-
-void settings::setEngineDefaultDownloadOptions( const QString& engineName,const QString& options )
-{
-	auto m = "EngineDefaultDownloadingOptions_" + engineName ;
-
-	m_settings.setValue( m,options ) ;
-}
-
 QString settings::cookieFilePath( const QString& engineName )
 {
 	auto m = "CookieFilePath_" + engineName ;
@@ -487,42 +668,43 @@ void settings::setCookieFilePath( const QString& engineName,const QString& cooki
 	m_settings.setValue( "CookieFilePath_" + engineName,cookieFilePath ) ;
 }
 
-void settings::setTheme( QApplication& app )
+void settings::setTheme( QApplication& app,const QString& themeBasePath )
 {
-	settings::darkModes darkModes( this->darkMode() ) ;
+	themes ths( this->themeName(),themeBasePath ) ;
 
-	if( darkModes.darkModeIsSet() ){
+	if( !QFile::exists( themeBasePath ) ){
 
-		if( darkModes.fusionTheme() ){
+		QDir().mkpath( themeBasePath ) ;
 
-			app.setStyle( QStyleFactory::create( "Fusion" ) ) ;
-			QPalette darkPalette ;
-			QColor darkColor = QColor( 45,45,45 ) ;
-			QColor disabledColor = QColor( 127,127,127 ) ;
-			darkPalette.setColor( QPalette::Window,darkColor ) ;
-			darkPalette.setColor( QPalette::WindowText,Qt::white ) ;
-			darkPalette.setColor( QPalette::Base,QColor( 18,18,18 ) ) ;
-			darkPalette.setColor( QPalette::AlternateBase,darkColor ) ;
-			darkPalette.setColor( QPalette::ToolTipBase,Qt::white ) ;
-			darkPalette.setColor( QPalette::ToolTipText,Qt::white ) ;
-			darkPalette.setColor( QPalette::Text,Qt::white ) ;
-			darkPalette.setColor( QPalette::Disabled,QPalette::Text,disabledColor ) ;
-			darkPalette.setColor( QPalette::Button,darkColor ) ;
-			darkPalette.setColor( QPalette::ButtonText,Qt::white ) ;
-			darkPalette.setColor( QPalette::Disabled,QPalette::ButtonText,disabledColor ) ;
-			darkPalette.setColor( QPalette::BrightText,Qt::red) ;
-			darkPalette.setColor( QPalette::Link,QColor( 42,130,218 ) ) ;
-			darkPalette.setColor( QPalette::Highlight,QColor( 42,130,218 ) ) ;
-			darkPalette.setColor( QPalette::HighlightedText,Qt::black ) ;
-			darkPalette.setColor( QPalette::Disabled,QPalette::HighlightedText,disabledColor ) ;
+		auto defaultThemePath = ths.defaultthemeFullPath() ;
 
-			app.setPalette( darkPalette ) ;
+		if( !QFile::exists( defaultThemePath ) ){
 
-			app.setStyleSheet("QToolTip { color: #ffffff; background-color: #2a82da; border: 1px solid white; }");
+			QFile f( defaultThemePath ) ;
+
+			if( f.open( QIODevice::WriteOnly ) ){
+
+				f.write( QJsonDocument( ths.defaultTheme() ).toJson( QJsonDocument::Indented ) ) ;
+			}
+		}
+	}
+
+	if( ths.usingThemes() ){
+
+		QFile f( ths.themeFullPath() ) ;
+
+		if( !f.open( QIODevice::ReadOnly ) ){
+
+			ths.setDefaultTheme( app ) ;
 		}else{
-			QFile file( darkModes.themeFileName() ) ;
-			file.open( QFile::ReadOnly ) ;
-			app.setStyleSheet( file.readAll() ) ;
+			auto obj = QJsonDocument::fromJson( f.readAll() ).object() ;
+
+			if( obj.isEmpty() ){
+
+				return ths.setDefaultTheme( app ) ;
+			}
+
+			ths.setTheme( app,obj ) ;
 		}
 	}
 }
@@ -650,6 +832,21 @@ void settings::setHighDpiScalingFactor( const QString& m )
 	m_settings.setValue( "EnabledHighDpiScalingFactor",m.toUtf8() ) ;
 }
 
+QString settings::textEncoding()
+{
+	if( !m_settings.contains( "YtDlpTextEncoding" ) ){
+
+		m_settings.setValue( "YtDlpTextEncoding",QString() ) ;
+	}
+
+	return m_settings.value( "YtDlpTextEncoding" ).toString() ;
+}
+
+void settings::setTextEncoding( const QString& e )
+{
+	m_settings.setValue( "YtDlpTextEncoding",e ) ;
+}
+
 void settings::setlibraryDownloadFolder( const QString& e )
 {
 	m_settings.setValue( "LibraryDownloadFolder",QDir::fromNativeSeparators( e ) ) ;
@@ -757,14 +954,9 @@ QStringList settings::localizationLanguages()
 	return m ;
 }
 
-QString settings::configPaths()
+const QString& settings::configPaths()
 {
-	if( settings::portableVersion() ){
-
-		return settings::portableVersionConfigPath() ;
-	}else{
-		return _configPath() ;
-	}
+	return m_dataPath ;
 }
 
 QString settings::commandOnSuccessfulDownload()
@@ -824,18 +1016,14 @@ void settings::setLastUsedOption( const QString& m,const QString& e,settings::ta
 
 QString settings::localizationLanguagePath()
 {
-	if( m_portableVersion ){
+	if( utility::platformIsWindows() ){
 
-		return QDir().currentPath() + "/translations" ;
+		return m_exePath + "/translations" ;
 	}
 
 	if( !m_settings.contains( "TranslationsPath" ) ){
 
-		if( utility::platformIsWindows() ){
-
-			m_settings.setValue( "TranslationsPath",QDir().currentPath() + "/translations" ) ;
-
-		}else if( utility::platformIsOSX() ){
+		if( utility::platformIsOSX() ){
 
 			m_settings.setValue( "TranslationsPath",TRANSLATION_PATH ) ;
 		}else{
