@@ -28,6 +28,7 @@
 #include <QTimer>
 #include <QThread>
 #include <QDebug>
+#include <QTime>
 
 #include <type_traits>
 #include <memory>
@@ -59,7 +60,10 @@ namespace utility
 	class debug
 	{
 	public:
-		debug( const QString& e = QString() ) :
+		debug() : m_switch( "--debug" )
+		{
+		}
+		debug( const QString& e ) :
 			m_switch( e )
 		{
 		}
@@ -71,6 +75,7 @@ namespace utility
 		debug& operator<<( const QString& e ) ;
 		debug& operator<<( const QByteArray& e ) ;
 		debug& operator<<( const QStringList& e ) ;
+		debug& operator<<( const QList<QByteArray>& e ) ;
 	private:
 		template< typename T >
 		debug& _print( const T& e )
@@ -118,24 +123,77 @@ namespace utility
 		{
 			return "${CommandName}" ;
 		}
+		static QString postProcessMarker()
+		{
+			return "DoneDownloading" ;
+		}
+		template< typename T >
+		static QByteArray doneDownloadingText( const T& p )
+		{
+			if( p.cancelled() ){
+
+				return "[media-downloader] Download Cancelled" ;
+
+			}else if( p.success() ){
+
+				return "[media-downloader] Download Completed Successfully" ;
+
+			}else if( p.exitStatus() == QProcess::NormalExit ){
+
+				auto m = QString::number( p.exitCode() ).toUtf8() ;
+
+				return "[media-downloader] Download Failed(ErrorCode=" + m + ")" ;
+			}else{
+				return "[media-downloader] Download Failed, Engine crashed" ;
+			}
+		}
+		static bool doneDownloadingText( const QByteArray& e )
+		{
+			return e.startsWith( "[media-downloader] Download " ) ;
+		}
+		static QString uploadDate()
+		{
+			return QObject::tr( "Upload Date:" ) ;
+		}
+		static QString duration()
+		{
+			return QObject::tr( "Duration:" ) ;
+		}
+		static QString engineName()
+		{
+			return QObject::tr( "Engine Name:" ) + " " ;
+		}
+		static QString subtitle()
+		{
+			return QObject::tr( "Subtitle Name" ) ;
+		}
+		static QString downloadOptions()
+		{
+			return QObject::tr( "Download Options" ) ;
+		}
+		static QString downloadExtendedOptions()
+		{
+			return QObject::tr( "Extra Downloading Options" ) ;
+		}
+		static QString downloadTimeInterval()
+		{
+			return QObject::tr( "Download Time Interval" ) ;
+		}
+		static QString downloadChapters()
+		{
+			return QObject::tr( "Download Chapters" ) ;
+		}
+		static QString splitByChapters()
+		{
+			return QObject::tr( "Split By Chapters" ) ;
+		}
 	private:
 	};
 
 	class args
 	{
 	public:
-		args( const QString& e )
-		{
-			if( !e.isEmpty() ){
-
-				m_otherOptions = util::splitPreserveQuotes( e ) ;
-
-				if( !m_otherOptions.isEmpty() ){
-
-					m_quality = m_otherOptions.takeFirst() ;
-				}
-			}
-		}
+		args( const QString& e,const engines::engine& ) ;
 		const QString& quality() const
 		{
 			return m_quality ;
@@ -239,46 +297,146 @@ namespace utility
 		}
 	}
 
+	class cliArguments
+	{
+	public:
+		cliArguments( int argc,char ** argv ) : m_argc( argc ),m_argv( argv )
+		{
+		}
+		bool contains( const char * ) const ;
+		bool runningUpdated() const ;
+		bool portable() const ;
+		QString dataPath() const ;
+		QString originalPath() const ;
+		QString originalVersion() const ;
+		QString value( const char * ) const ;
+		QStringList arguments( const QString&,const QString&,bool ) const ;
+	private:
+		int m_argc ;
+		char ** m_argv ;
+	} ;
+
+	void setDefaultEngine( const Context& ctx,const QString& name );
 	const engines::engine& resolveEngine( const tableWidget&,
 					      const engines::engine&,
 					      const engines& engines,
 					      int row ) ;
 
 	QString failedToFindExecutableString( const QString& cmd ) ;
+	int sequentialID() ;
 	int concurrentID() ;
-	void saveDownloadList( const Context&,QMenu&,tableWidget& ) ;
+	void initDone() ;
+	void saveDownloadList( const Context&,QMenu&,tableWidget&,bool ) ;
+	void saveDownloadList( const Context&,tableWidget&,bool ) ;
 	void wait( int time ) ;
 	void waitForOneSecond() ;
 	void openDownloadFolderPath( const QString& ) ;
+	void setPermissions( QFile& ) ;
+	void setPermissions( const QString& ) ;
+	QString runningVersionOfMediaDownloader() ;
 	QString homePath() ;
 	QString python3Path() ;
 	QString clipboardText() ;
+	QString fromSecsSinceEpoch( qint64 ) ;
+	struct downLoadOptions
+	{
+		downLoadOptions()
+		{
+		}
+		downLoadOptions( QString e ) : downloadOptions( std::move( e ) )
+		{
+		}
+		bool hasExtraOptions = false ;
+		QString downloadOptions ;
+	} ;
+	utility::downLoadOptions setDownloadOptions( const engines::engine&,tableWidget&,int,const QString& ) ;
 	bool platformIsWindows() ;
-	bool platformIs32BitWindows() ;
+	bool platformIs32Bit() ;
 	bool platformIsLinux() ;
 	bool platformIsOSX() ;
+	bool platformisOS2() ;
 	bool platformIsNOTWindows() ;
+	bool platformIsLikeWindows() ;
+	QString windowsApplicationDirPath() ;
 	bool isRelativePath( const QString& ) ;
 	QString downloadFolder( const Context& ctx ) ;
-	const QProcessEnvironment& processEnvironment( const Context& ctx ) ;
+	bool onlyWantedVersionInfo( const utility::cliArguments& ) ;
+	bool startedUpdatedVersion( settings&,const utility::cliArguments& ) ;
+
+	class addJsonCmd
+	{
+	public:
+		struct entry
+		{
+			QString platform ;
+			struct arch{
+				QString archName ;
+				QString exeName ;
+				QStringList exeArgs ;
+			} ;
+			std::vector< arch > platformData ;
+		} ;
+		addJsonCmd( QJsonObject& obj ) :
+			m_mainObj( obj )
+		{
+		}
+		void add( const addJsonCmd::entry& e ) ;
+		void done()
+		{
+			m_mainObj.insert( "Cmd",m_obj ) ;
+		}
+	private:
+		QJsonObject& m_mainObj ;
+		QJsonObject m_obj ;
+	};
 
 	class locale
 	{
 	public:
 		QString formattedDataSize( qint64 ) const ;
+		static QString secondsToString( int s )
+		{
+			if( s < 3600 ){
+
+				return QTime( 0,0,0,0 ).addSecs( s ).toString( "mm:ss" ) ;
+			}else{
+				return QTime( 0,0,0,0 ).addSecs( s ).toString( "hh:mm:ss" ) ;
+			}
+		}
 	private:
 		QLocale m_locale ;
 	};
 
+	class uiIndex
+	{
+	public:
+		uiIndex() : m_index( 0 ),m_total( 0 )
+		{
+		}
+		uiIndex( int index,int total ) :
+			m_index( index ),m_total( total )
+		{
+		}
+
+		QString toString( bool,const QStringList& ) const ;
+	private:
+		QString toString( int ) const ;
+
+		int m_index ;
+		int m_total ;
+	} ;
+
 	struct updateOptionsStruct
 	{
+		const utility::downLoadOptions& dopts ;
 		const engines::engine& engine ;
-		const engines::enginePaths& enginePaths ;
 		settings& stts;
 		const utility::args& args ;
-		const QString& indexAsString ;
+		const utility::uiIndex& uiIndex ;
 		bool forceDownload ;
 		const QStringList& urls ;
+		const tableWidget::entry& tableEntry ;
+		const Context& ctx ;
 	};
 
 	QStringList updateOptions( const updateOptionsStruct& ) ;
@@ -546,7 +704,7 @@ namespace utility
 	template< typename WhenDone,typename WithData >
 	void run( const QString& cmd,const QStringList& args,WhenDone w,WithData p )
 	{
-		util::run( cmd,args,[]( QProcess& ){},std::move( w ),std::move( p ) ) ;
+		utils::qprocess::run( cmd,args,[]( QProcess& ){},std::move( w ),std::move( p ) ) ;
 	}
 
 	template< typename Function,typename FunctionConnect >
@@ -669,54 +827,13 @@ namespace utility
 		}
 	}
 
-	class versionInfo : public QObject
-	{
-		Q_OBJECT
-	public:
-		~versionInfo() override ;
-
-		versionInfo( Ui::MainWindow& ui ) : m_ui( ui )
-		{
-		}
-		void setContext( const Context& ctx )
-		{
-			m_ctx = &ctx ;
-			m_networkAccess = ctx ;
-		}
-		template< typename Then >
-		void setVersion( const engines::engine& engine,Then then ){
-
-			engines::engine::exeArgs::cmd cmd( engine.exePath(),{ engine.versionArgument() } ) ;
-
-			util::run( cmd.exe(),cmd.args(),[ &engine,then = std::move( then ) ]( const util::run_result& r ){
-
-				if( r.success() ){
-
-					engine.versionString( r.stdOut ) ;
-				}
-
-				then( r.success() ) ;
-
-			},QProcess::ProcessChannelMode::MergedChannels ) ;
-		}
-		void check( const engines::Iterator& iter ) ;
-		networkAccess& network()
-		{
-			return m_networkAccess.get() ;
-		}
-	signals:
-		void vinfoDone() ;
-	private:
-		void printEngineVersionInfo( const engines::Iterator& iter ) ;
-		const Context * m_ctx ;
-		util::storage< networkAccess > m_networkAccess ;
-		Ui::MainWindow& m_ui ;
-	};
-
 	class ProcessExitState
 	{
 	public:
-		ProcessExitState( bool c,int s,int d,QProcess::ExitStatus e ) :
+		ProcessExitState()
+		{
+		}
+		ProcessExitState( bool c,int s,qint64 d,QProcess::ExitStatus e ) :
 			m_cancelled( c ),
 			m_exitCode( s ),
 			m_duration( d ),
@@ -739,15 +856,15 @@ namespace utility
 		{
 			return m_exitCode == 0 && m_exitStatus == QProcess::ExitStatus::NormalExit ;
 		}
-		int duration() const
+		qint64 duration() const
 		{
 			return m_duration ;
 		}
 	private:
 		bool m_cancelled = false ;
-		int m_exitCode ;
-		int m_duration ;
-		QProcess::ExitStatus m_exitStatus ;
+		int m_exitCode = 255 ;
+		qint64 m_duration = 0 ;
+		QProcess::ExitStatus m_exitStatus = QProcess::ExitStatus::NormalExit ;
 	};
 
 	class ProcessOutputChannels
@@ -849,13 +966,25 @@ namespace utility
 
 			m_timer->stop() ;
 
+			auto m = m_timeCounter.elapsedTime() ;
+
+			ProcessExitState state( m_cancelled,s,m,std::move( e ) ) ;
+
 			if( m_options.listRequested() ){
 
-				m_options.listRequested( std::move( m_data ) ) ;
+				m_options.listRequested( state,std::move( m_data ) ) ;
 			}
 
-			auto m = m_timeCounter.elapsedTime() ;
-			m_options.done( ProcessExitState( m_cancelled,s,m,std::move( e ) ) ) ;
+			m_logger.add( [ &,this ]( Logger::Data& e,int id,bool s ){
+
+				auto d = utility::stringConstants::doneDownloadingText( state ) ;
+
+				m_engine.processData( e,d,id,s ) ;
+			} ) ;
+
+			m_options.done( std::move( state ) ) ;
+
+			m_logger.registerDone() ;
 		}
 		void withData( QProcess::ProcessChannel channel,const QByteArray& data )
 		{
@@ -890,11 +1019,21 @@ namespace utility
 
 				auto c = m_channels.channel() ;
 
-				if( c == QProcess::ProcessChannel::StandardOutput && c == channel ){
+				if( channel == QProcess::ProcessChannel::StandardOutput ){
 
-					_withData( data ) ;
+					if( c == QProcess::ProcessChannel::StandardOutput ){
+
+						_withData( data ) ;
+					}else{
+						//??
+					}
 				}else{
-					m_logger.logError( data ) ;
+					if( c == QProcess::ProcessChannel::StandardError ){
+
+						_withData( data ) ;
+					}else{
+						m_logger.logError( data ) ;
+					}
 				}
 			}
 		}
@@ -927,7 +1066,7 @@ namespace utility
 		       Options options,
 		       Tlogger logger,
 		       Connection conn,
-		       utility::ProcessOutputChannels channels = utility::ProcessOutputChannels() )
+		       utility::ProcessOutputChannels channels )
 	{
 		using ctx = utility::context< Tlogger,Options,Connection > ;
 
@@ -941,7 +1080,7 @@ namespace utility
 
 		if( cmd.valid() ){
 
-			util::run( cmd.exe(),cmd.args(),[ &cmd,ctx = std::move( ctx ) ]( QProcess& exe )mutable{
+			utils::qprocess::run( cmd.exe(),cmd.args(),[ &cmd,ctx = std::move( ctx ) ]( QProcess& exe )mutable{
 
 				ctx.whenCreated( exe,cmd ) ;
 
@@ -969,8 +1108,8 @@ namespace utility
 	class reverseIterator
 	{
 	public:
-		typedef typename std::remove_reference_t< std::remove_cv_t< List > > ::value_type value_type ;
-		typedef typename std::remove_reference_t< std::remove_cv_t< List > > ::size_type size_type ;
+		typedef typename std::remove_reference_t< std::remove_cv_t< List > >::value_type value_type ;
+		typedef typename std::remove_reference_t< std::remove_cv_t< List > >::size_type size_type ;
 
 	        reverseIterator( List s ) :
 		        m_list( s ),
@@ -987,9 +1126,11 @@ namespace utility
 		}
 		auto& next()
 		{
-			auto s = static_cast< typename reverseIterator< List >::size_type >( m_index-- ) ;
-
-			return m_list[ s ] ;
+			return m_list[ this->nextValue() ] ;
+		}
+		auto nextAsValue()
+		{
+			return m_list[ this->nextValue() ] ;
 		}
 		template< typename Function,
 			  util::types::has_bool_return_type< Function,typename reverseIterator< List >::value_type > = 0 >
@@ -997,7 +1138,7 @@ namespace utility
 		{
 			while( this->hasNext() ){
 
-				if( function( this->next() ) ){
+				if( function( m_list[ this->nextValue() ] ) ){
 
 					break ;
 				}
@@ -1009,10 +1150,14 @@ namespace utility
 		{
 			while( this->hasNext() ){
 
-				function( this->next() ) ;
+				function( m_list[ this->nextValue() ] ) ;
 			}
 		}
 	private:
+		auto nextValue()
+		{
+			return static_cast< typename reverseIterator< List >::size_type >( m_index-- ) ;
+		}
 		List m_list ;
 		int m_index ;
 	} ;
@@ -1025,19 +1170,23 @@ namespace utility
 	class MediaEntry
 	{
 	public:
+		MediaEntry()
+		{
+		}
 		MediaEntry( const QString& url ) :
 			m_url( url ),
 			m_json( QByteArray() )
 		{
 		}
-		MediaEntry( const QString& uiText,const QString& url ) :
-			m_title( uiText ),
+		MediaEntry( const QString& title,const QString& url ) :
+			m_title( title ),
 			m_url( url ),
 			m_json( QByteArray() )
 		{
 		}
 
-		MediaEntry( const QString& url,const QByteArray& data ) ;
+		MediaEntry( const QByteArray& data ) ;
+
 		QString uiText() const ;
 
 		const QString& thumbnailUrl() const
@@ -1076,10 +1225,47 @@ namespace utility
 		{
 			return m_id ;
 		}
+		const QJsonArray& formats() const
+		{
+			return m_formats ;
+		}
+		QJsonObject uiJson() const ;
 		int intDuration() const
 		{
 			return m_intDuration ;
 		}
+		const QString& playlist() const
+		{
+			return m_playlist ;
+		}
+		const QString& playlist_uploader() const
+		{
+			return m_playlist_uploader ;
+		}
+		const QString& playlist_count() const
+		{
+			return m_playlist_count ;
+		}
+		const QString& playlist_id() const
+		{
+			return m_playlist_id ;
+		}
+		const QString& playlist_title() const
+		{
+			return m_playlist_title ;
+		}
+		const QString& playlist_uploader_id() const
+		{
+			return m_playlist_uploader_id ;
+		}
+		const QString& n_entries() const
+		{
+			return m_n_entries ;
+		}
+		MediaEntry move() const
+		{
+			return std::move( *const_cast< MediaEntry * >( this ) ) ;
+		}
 	private:
 		QString m_thumbnailUrl ;
 		QString m_title ;
@@ -1087,8 +1273,86 @@ namespace utility
 		QString m_url ;
 		QString m_duration ;
 		QString m_id ;
+		QString m_uploader ;
+		QString m_playlist ;
+		QString m_playlist_count ;
+		QString m_playlist_id ;
+		QString m_playlist_title ;
+		QString m_playlist_uploader ;
+		QString m_playlist_uploader_id ;
+		QString m_n_entries ;
+
+		QJsonArray m_formats ;
 		int m_intDuration ;
 		util::Json m_json ;
+	} ;
+
+	class networkReply
+	{
+	public:
+		networkReply()
+		{
+		}
+		networkReply( const Context& ctx,const utils::network::reply& reply )
+		{
+			this->getData( ctx,reply ) ;
+		}
+		networkReply( QObject * obj,
+			      const char * member,
+			      tableWidget * t,
+			      int id,
+			      utility::MediaEntry m ) :
+			m_id( id ),
+			m_mediaEntry( std::move( m ) ),
+			m_table( t )
+		{
+			this->invoke( obj,member ) ;
+		}
+		networkReply( QObject * obj,
+			      const char * member,
+			      const Context& ctx,
+			      const utils::network::reply& reply,
+			      tableWidget * t,
+			      int id,
+			      utility::MediaEntry m ) :
+			m_id( id ),
+			m_mediaEntry( std::move( m ) ),
+			m_table( t )
+		{
+			this->getData( ctx,reply ) ;
+			this->invoke( obj,member ) ;
+		}
+		const QByteArray& data() const
+		{
+			return m_data ;
+		}
+		bool success() const
+		{
+			return !m_data.isEmpty() ;
+		}
+		int id() const
+		{
+			return m_id ;
+		}
+		int index() const
+		{
+			return m_id ;
+		}
+		const utility::MediaEntry& media() const
+		{
+			return m_mediaEntry ;
+		}
+		tableWidget& table() const
+		{
+			return *m_table ;
+		}
+	private:
+		void invoke( QObject *,const char * ) ;
+		void getData( const Context& ctx,const utils::network::reply& ) ;
+		QByteArray m_data ;
+		int m_id ;
+		utility::MediaEntry m_mediaEntry ;
+		tableWidget * m_table ;
 	};
 
 	template< typename FinishedState >
@@ -1117,7 +1381,7 @@ namespace utility
 				engine.runCommandOnDownloadedFile( a,backUpUrl ) ;
 			}
 
-			if( f.allFinished() ){
+			if( f.done() ){
 
 				auto a = s.commandWhenAllFinished() ;
 
@@ -1137,7 +1401,8 @@ namespace utility
 	class options
 	{
 	public:
-		options( Opts opts,Functions functions ) :
+		options( const engines::engine& engine,Opts opts,Functions functions ) :
+			m_engine( engine ),
 			m_opts( std::move( opts ) ),
 			m_functions( std::move( functions ) )
 		{
@@ -1146,9 +1411,9 @@ namespace utility
 		{
 			m_functions.done( std::move( e ),m_opts ) ;
 		}
-		void listRequested( QByteArray e )
+		void listRequested( const utility::ProcessExitState& s,QByteArray e )
 		{
-			m_functions.list( std::move( e ) ) ;
+			m_functions.list( s,std::move( e ) ) ;
 		}
 		bool listRequested()
 		{
@@ -1172,9 +1437,10 @@ namespace utility
 		}
 		const QProcessEnvironment& processEnvironment() const
 		{
-			return utility::processEnvironment( m_opts.ctx ) ;
+			return m_engine.processEnvironment() ;
 		}
 	private:
+		const engines::engine& m_engine ;
 		Opts m_opts ;
 		Functions m_functions ;
 	} ;
@@ -1196,7 +1462,7 @@ namespace utility
 	template< typename DisableAll,typename Done >
 	auto OptionsFunctions( DisableAll disableAll,Done done )
 	{
-		auto aa = []( QByteArray ){} ;
+		auto aa = []( const utility::ProcessExitState&,QByteArray ){} ;
 
 		using type = Functions< decltype( aa ),DisableAll,Done > ;
 
@@ -1204,4 +1470,6 @@ namespace utility
 	}
 }
 
+Q_DECLARE_METATYPE( utility::networkReply )
+
 #endif
